from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Dataset, ImanRelation, BOMLine
from tcsoa.gen.Internal.Visualization._2008_06.DataManagement import DatasetInfo, NamedRefsInDataset, NamedRefUploadOrUpdateInfo
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindNodesInProductViewResultResponse(TcBaseObj):
    """
    Returns the nodes present in the product view.
    
    :var outputNodes: Nodes which are present in the product view out of the input node vector.The nodes can be either
    BOM line(s), BOP line(s) or an Item(s).
    :var serviceData: Standard service data to handle partial errors.
    """
    outputNodes: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class FindProductViewForNodesResult(TcBaseObj):
    """
    Hold the result of the search.
    
    :var productView: The product view which contains the nodes or set of the nodes which the user is searching for.
    :var nodesPresentInPv: This is the set of nodes out of the input nodes which are present in the product view.The
    nodes can be either BOM line(s), BOP line(s) or an Item(s).
    :var pvAttachedTo: The line(BOM or BOP) in the product or process structure to which the PV is attached.
    """
    productView: Dataset = None
    nodesPresentInPv: List[BusinessObject] = ()
    pvAttachedTo: BusinessObject = None


@dataclass
class FindProductViewForNodesResultRespose(TcBaseObj):
    """
    Contains the result and the errors of the product view search
    
    :var productViews: Returns the result of the search.
    :var serviceData: This is standard service data which contains the partial errors
    """
    productViews: List[FindProductViewForNodesResult] = ()
    serviceData: ServiceData = None


@dataclass
class GatherSnapshot3DInput(TcBaseObj):
    """
    Input structure for GatherSnapshot3DInput
    
    :var clientId: Client specified identifier to help keep track of the objects returned.
    :var bomLine: Object reference of the input BOMLine object.  Typically the top line object is specified, but caller
    can also include lines where Product Views may be attached depending on the filtering behavior desired.
    :var refType: GRM relation type to be followed from the specified bomline for finding the SnapShotViewData
    datasets.  The VisTopLevelRef refType should be specified for the topline to find all Product Views authored in
    this context.  The IMAN_3D_snap_shot relation should be specified along with a bomline where a Product View must be
    attached in order to filter by attachment location.
    """
    clientId: str = ''
    bomLine: BOMLine = None
    refType: RefType = None


@dataclass
class GatherSnapshot3DListOutputPreview(TcBaseObj):
    """
    A List of Snapshot3DPreview objects.
    
    :var snapshotList: A List of Snapshot3DPreview objects.
    """
    snapshotList: List[Snapshot3DOutput] = ()


@dataclass
class GatherSnapshot3DListResponse(TcBaseObj):
    """
    Response containing a map between BOMLine and 'Snapshot3DPreviewList' which provides the minimum information
    necessary to populate the gallery about the snapshots found.
    
    :var snapshot3DOutputMap: A map between 'BOMLineUid' string and 'GatherSnapshot3DListOutputPreview'.
    :var serviceData: Standard  ServiceData member.
    """
    snapshot3DOutputMap: GatherSnapshot3DListOutputMap = None
    serviceData: ServiceData = None


@dataclass
class NewSnapshot3DInput(TcBaseObj):
    """
    Input structure for the 'createSnapshot3D' operation that creates the Product View (SnapShotViewData) Dataset.
    
    :var clientId: Identifier that helps the client to track the objects returned.
    :var attachToBOMLine: 'BOMLineInput' object reference to attach the new Product View Dataset.
    :var datasetInfo: Object of 'DatasetInfo', containing Dataset creation info.
    :var visibleLinesList: A list of visible BOMLine objects.
    :var createStructureFile: True if structure PLMXML file is to be generated.
    :var namedRefFileUpdateInfoList: (Optional) List of 'nameRefFiles' to be attached to Product View Dataset at the
    time of creation.  (named references in the Dataset).
    """
    clientId: str = ''
    attachToBOMLine: BOMLine = None
    datasetInfo: DatasetInfo = None
    visibleLinesList: List[BOMLine] = ()
    createStructureFile: bool = False
    namedRefFileUpdateInfoList: List[NamedRefUploadOrUpdateInfo] = ()


@dataclass
class SearchCriteria(TcBaseObj):
    """
    This contains all the required inputs for the product search
    
    :var searchScope: Scope lines within which the search will be performed. The scope lines can be either BOM or BOP
    lines.
    :var nodesToSearch: A list of nodes for which user is interested to find in the product views which are associated
    with the search scope. The scope lines can be either BOM lines , BOP lines ot Items.
    :var nodeCombination: Allows the user to specify the combination of node for the search. Currently only two value
    are upported : OR and AND. If the value is OR,  the use wants to find the PVs which include any of the node from
    the above node list. If it is AND, the user is intended to find the PVs which include all the nodes from the above
    list.If the user has not provided any input, the nodeCombination parameter value will be OR.
    """
    searchScope: List[BusinessObject] = ()
    nodesToSearch: List[BusinessObject] = ()
    nodeCombination: str = ''


@dataclass
class Snapshot3DInfoInput(TcBaseObj):
    """
    Input structure for getting SnapShotViewData Dataset info.
    
    :var clientId: Identifier for client data.
    :var snapshot3DDataset: Tag of the input SnapShotViewData Dataset.
    :var getStaticFilesInfo: If true, it will return the static information of this Product View.  This may not be
    available if a static representation of the Product View was not saved, or the data model does not support it.
    """
    clientId: str = ''
    snapshot3DDataset: Dataset = None
    getStaticFilesInfo: bool = False


@dataclass
class Snapshot3DInfoOutput(TcBaseObj):
    """
    Output structure for the info contained in the Snapshot 3D.
    
    :var namedRefFileInfoList: List of file info named referenced by the SnapShotViewData.
    :var relObjList: List of objects related to SnapShotViewData via GRM.
    :var snapshot3DVisibleLines: List of visible lines in the Product View.
    """
    namedRefFileInfoList: List[NamedRefsInDataset] = ()
    relObjList: List[Snapshot3DRelObjInfo] = ()
    snapshot3DVisibleLines: Snapshot3DVisibleLines = None


@dataclass
class Snapshot3DInfoResponse(TcBaseObj):
    """
    Return structure for 'Snapshot3DInfo'.
    
    :var snapshot3DInfoMap: A map containing client ids and output structure as key/value pairs. The keys corresponding
    to the clientId specified in the 'Snapshot3DInfoInput'. The values are a struct of 'Snapshot3DInfoOutput' data.
    :var serviceData: Standard 'ServiceData' member.
    """
    snapshot3DInfoMap: Snapshot3DInfoOutputMap = None
    serviceData: ServiceData = None


@dataclass
class Snapshot3DOutput(TcBaseObj):
    """
    Output structure for the info contained in the snapshot3D dataset.
    
    :var snapshot3DDataset: The Snapshot3D dataset object gathered.
    :var namedRefFileInfoList: Named References in dataset; only thumbnail file and preview file named references will
    be returned in case of gatherSnapshot3DList SOA call.
    """
    snapshot3DDataset: Dataset = None
    namedRefFileInfoList: List[NamedRefsInDataset] = ()


@dataclass
class Snapshot3DRelObjInfo(TcBaseObj):
    """
    List of objects related to SnapShotViewData via GRM.
    
    :var relType: The relationship type namedRef or GRM.
    :var relObj: Object Reference attached to the Dataset.
    :var relation: GRM relation on the attached Dataset such as VISTopLevelRef or IMAN_3D_snap_shot. For namedRef, it
    is RootContext.
    """
    relType: RelType = None
    relObj: BusinessObject = None
    relation: ImanRelation = None


@dataclass
class Snapshot3DStructureFilesInput(TcBaseObj):
    """
    Input Structure for updating the PLMXML structure file.
    
    :var clientId: Identifier for client data
    :var snapshot3DDataset: SnapshotDataset3D Dataset for which strucuture file to be updated.
    :var visibleLinesList: The list of visible BOMLine objects.  The ConfigurationContext object will be updated if
    needed.
    """
    clientId: str = ''
    snapshot3DDataset: Dataset = None
    visibleLinesList: List[BOMLine] = ()


@dataclass
class Snapshot3DUpdateInput(TcBaseObj):
    """
    Input structure for updating the SnapShotViewData Dataset.
    
    :var clientId: A unique ID used to help the client keep track of the SnapShotViewData Datasets in the output when
    using the call in batch mode when multiple Datasets are submitted at once.  If more than one Dataset tag is used,
    it is important to use unique 'clientId' strings in the input since they are used as keys in the return output map.
     If the same 'clientId' string is used, information will be lost.
    :var snapshot3DDataset: Tag of the  SnapShotViewData Dataset to be updated.
    :var attachToBOMLine: 'BOMLineInput' object reference that controls where (to which ItemRevision) the updated
    SnapShotViewData Dataset is attached.  If not specified the update operation will attach the Dataset to the same
    place the previous Dataset was attached.
    :var createStructureFile: A Boolean used to control whether or not a new structure PLMXML file is exported.  True
    to generate updated PLMXML, False if not.  If PLMXML is already in the original Dataset, it will be removed if
    there are visible lines sent with this call.
    :var updateVisibleLinesList: Set to True to force the Visible Lines to get updated.  This allows the special case
    where a snapshot is updated with no visible lines.
    :var visibleLinesList: Used if 'updateVisibleLinesList' is set to True.  Results in reexport of the structure
    PLMXML and update of it in Dataset if 'createStructureFile' is true.  If 'createStructureFile' is false, the old
    PLMXML will still be removed.  The ConfigurationContext object will be updated if needed.
    :var namedRefFileInfoList: List of 'FileInfo' for named referenced by 'snapshot3D'.  All the old named ref files
    will be removed, and completely replaced with these which may result in fewer named ref files.  If the old named
    ref should be kept (e.g. an old image preview that was not regenerated) it must be passed in this list.
    """
    clientId: str = ''
    snapshot3DDataset: Dataset = None
    attachToBOMLine: BOMLine = None
    createStructureFile: bool = False
    updateVisibleLinesList: bool = False
    visibleLinesList: List[BOMLine] = ()
    namedRefFileInfoList: List[NamedRefUploadOrUpdateInfo] = ()


@dataclass
class Snapshot3DUpdateResponse(TcBaseObj):
    """
    Object containing list of Snapshot3DInfo objects of updated Product View Dataset.
    
    :var snapshot3DUpdateOutputMap: A map between 'clientIds' and  'Snapshot3DUpdateOutput'.
    :var serviceData: Standard service data member.
    """
    snapshot3DUpdateOutputMap: Snapshot3DUpdateOutputMap = None
    serviceData: ServiceData = None


@dataclass
class Snapshot3DUpdateStructureFilesResponse(TcBaseObj):
    """
    SOA Response containing the map between 'clientids' and SnapshotDataset3D Datasets.
    
    :var snapshot3DStrFilesOutputMap: A map between 'clientIds' and SnapshotDataset3D Datasets.
    :var serviceData: Standard  ServiceData member.
    """
    snapshot3DStrFilesOutputMap: Snapshot3DUpdateStructureFilesOutputMap = None
    serviceData: ServiceData = None


@dataclass
class Snapshot3DVisibleLines(TcBaseObj):
    """
    List of visible lines in the Product View.
    
    :var uidType: Type of the return uid string list.
    :var visibleLinesMap: Map of visible BOMLines.  The key is the 'parentRef' from another 'VisibleLine' struct.
    :var relDatasetInfoMap: This will hold the Dataset info related to the visible lines. The 'relDatasetRef' string in
    the VisibleLine references the keys in this map.
    """
    uidType: UIDType = None
    visibleLinesMap: Snapshot3DVisibleLinesMap = None
    relDatasetInfoMap: Snapshot3DRelDatasetInfoMap = None


@dataclass
class StaticFileInfo(TcBaseObj):
    """
    Structure to hold the file info from the static capture of the snapshot.
    
    :var datasetUID: Direct Model dataset UID for the visible line
    :var imanFileUID: Iman file UID.  If there are multiple files in the dataset, return the first one.
    :var originalFileName: Full file name plus extensions.
    """
    datasetUID: str = ''
    imanFileUID: str = ''
    originalFileName: str = ''


@dataclass
class VisibleLine(TcBaseObj):
    """
    Structure to hold the unique data about the Visible BOMLine.
    
    :var uidString: UID string of the Visible BOMLines, its type is specified by 'uidType'.
    :var relDatasetRef: This is a string mapping to a related Dataset if not empty. Use the string as a key to the
    'relDatasetInfo' member in 'Snapshot3DVisibleLines' structure.
    :var parentRef: This is the ID of the parentRef.
    """
    uidString: str = ''
    relDatasetRef: str = ''
    parentRef: str = ''


@dataclass
class CreateSnapshot3DResponse(TcBaseObj):
    """
    Response structure returned by the 'CreateSnapshot3D' operation that consists primarily of a mapping between
    Dataset UIDs and client IDs. Also contains error information in the 'serviceData'.
    
    :var snapshot3DOutputMap: A map containing client_id (std::string) and Product View Dataset (std::string) output
    structure as key/value pairs.
    :var serviceData: Standard ServiceData member.
    """
    snapshot3DOutputMap: CreateSnapshot3DOutputMap = None
    serviceData: ServiceData = None


class RefType(Enum):
    """
    GRM relation type to be followed to gether 3D snapshot datasets.
    Ref_Snapshot3d = IMAN_3D_snap_shot
    Ref_VisTopLevel = VISTopLevelRef
    """
    Ref_Snapshot3d = 'Ref_Snapshot3d'
    Ref_VisTopLevel = 'Ref_VisTopLevel'


class RelType(Enum):
    """
    The relationship type namedRef or GRM.
    """
    TYPE_NamedRef = 'TYPE_NamedRef'
    TYPE_GRM = 'TYPE_GRM'


class UIDType(Enum):
    """
    Type of the return uid string list.
    """
    TYPE_APN = 'TYPE_APN'
    TYPE_CS = 'TYPE_CS'


"""
A map between clientIds (std::string) and Snapshot3D Dataset as key/value pairs.
"""
CreateSnapshot3DOutputMap = Dict[str, Dataset]


"""
A map between BOMLineUid string and Snapshot3DPreviewList
"""
GatherSnapshot3DListOutputMap = Dict[str, GatherSnapshot3DListOutputPreview]


"""
A map containing client ids and output structure as key/value pairs. The keys corresponding to the clientId specified in the Snapshot3DInfoInput. The values are a struct of Snapshot3DInfoOutput data.
"""
Snapshot3DInfoOutputMap = Dict[str, Snapshot3DInfoOutput]


"""
This will hold the Dataset info related to the visible lines. The relDatasetRef string in the VisibleLine struct references the keys in this map.
"""
Snapshot3DRelDatasetInfoMap = Dict[str, StaticFileInfo]


"""
A map between clientIds and  Snapshot3DOutput.
"""
Snapshot3DUpdateOutputMap = Dict[str, Snapshot3DOutput]


"""
A map between clientIds and snapshot3D datasets
"""
Snapshot3DUpdateStructureFilesOutputMap = Dict[str, Dataset]


"""
Map of visible BOMLines.  The key is the parentRef from another VisibleLine struct.
"""
Snapshot3DVisibleLinesMap = Dict[str, VisibleLine]



from __future__ import annotations

from tcsoa.gen.Manufacturing._2018_11.StructureManagement import CopyRecursivelyInputInfo, PasteDuplicateStructureResponse, CopyRecursivelyResponse, PasteDuplicateInput
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def pasteDuplicateStructure(cls, pasteDuplicateInput: List[PasteDuplicateInput], copyRulesKey: str, notifyEvents: bool) -> PasteDuplicateStructureResponse:
        """
        This service operation clones the provided source BOMLine and paste the cloned line under the corresponding
        target BOMLine. Additionally, returns a structure which provides the mapping between the source BOMLine and the
        corresponding new child BOMLine.
        
        Use cases:
        A user can copy a source BOMLine and clone it under a target BOMLine using paste duplicate command.
        Applications like NX, might want to know the mapping between the source BOMLine and corresponding newly created
        BOMLine to check if there is any constraint defined within the source BOMLine which should be added to the
        created BOMLine.
        """
        return cls.execute_soa_method(
            method_name='pasteDuplicateStructure',
            library='Manufacturing',
            service_date='2018_11',
            service_name='StructureManagement',
            params={'pasteDuplicateInput': pasteDuplicateInput, 'copyRulesKey': copyRulesKey, 'notifyEvents': notifyEvents},
            response_cls=PasteDuplicateStructureResponse,
        )

    @classmethod
    def copyRecursively(cls, copyInput: List[CopyRecursivelyInputInfo]) -> CopyRecursivelyResponse:
        """
        This service operation clones the input BOMLine structure. It either creates a new structure or updates the
        same structure by creating the cloned structure under the input target BOMLine.
        
        Use cases:
        Following use cases are supported.
        &bull;    Use Case 1: User can open a structure, launch "From Template" dialog and clone the structure. User
        can also provide the configuration information like variant rule(s), revision rule and also can specify if a
        new structure should be created. Other cloning parameters like "Carry over suppressed lines", "Carry future
        effectivity" can also be specified.
        &bull;    Use Case 2: User can launch the "From Template" dialog, provide the item ID of the structure and
        clone it. Similar to use case 1, user can specify the configuration information.
        """
        return cls.execute_soa_method(
            method_name='copyRecursively',
            library='Manufacturing',
            service_date='2018_11',
            service_name='StructureManagement',
            params={'copyInput': copyInput},
            response_cls=CopyRecursivelyResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, BOMLine
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReplaceInContextParameter(TcBaseObj):
    """
    This structure provides a set of input values for the replace in context action.
    
    :var bomLine: bomLine
    :var itemRevision: the item used to replace the item in context.
    """
    bomLine: BOMLine = None
    itemRevision: ItemRevision = None



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Ai._2012_09.Ai import ProjectFilter
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindRequestsWithDependencyFilter(TcBaseObj):
    """
    structure that captures the filtering options for getting the Request Objects after considering dependency. This
    will include the AppInterface filter options too.
    
    :var aiQryParams: structure to capture the filter option on parent(s) ApplicationInterfaces of the RequestObject(s)
    :var requestStatus: vector of strings representing the statuses on the request to search for. Currently, the valid
    values are a combination of (case sensitive): Normal, Warning, Severe, Abort
    :var stateDescription: state description to use for searching for RequestObject.
    :var statusDescription: status message by which to filter for RequestObjects.
    :var customStrings: map of strings that have the custom key and value pair to search on.
    """
    aiQryParams: ProjectFilter = None
    requestStatus: List[str] = ()
    stateDescription: str = ''
    statusDescription: str = ''
    customStrings: StrToStrMap201312 = None


"""
string to string map.
"""
StrToStrMap201312 = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0ListsParamReqments, Fnd0ParamReqmentRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetStdNoteParamAndValues(TcBaseObj):
    """
    'GetStdNoteParamAndValues' structure defines the name and value pair of each parameter for
    Fnd0ParamReqmentRevision. Each Parameter has list of values. And 'assignedValue' has the value of parameter for
    Fnd0ParamReqmentRevision when it is in context.
    
    :var parameter: Parameter name. Example-Temperature
    :var values: List of values for given parameter. Example. For above parameter Temperature list of values can be :
    0, 10, 20, 30, 40
    :var assignedValue: Current value of Parameter. Example. From the list of values as mentioned for values elements,
    the assignedValue for any specific context with relation 'Fnd0ListsParamReqments' can be say "10" or "20" for
    selected parameter type.
    """
    parameter: str = ''
    values: List[str] = ()
    assignedValue: str = ''


@dataclass
class OpenStdNoteContents(TcBaseObj):
    """
    OpenStdNoteContents structure contains input structure provided to this operation StdNoteInput, note text of
    Fnd0ParamReqmentRevision object which user want to see/edit, body_text of Fnd0ParamReqmentRevision object, and map
    of the parameter and values for the Fnd0ParamReqmentRevision object.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var inputDetails: 'StdNoteInput' structure given as input this operation.
    :var parametersOnRelation: A list of 'GetStdNoteParamAndValues' which gives the list of parameters and their values
    for Fnd0ParamReqmentRevision object.
    :var noteText: Note Text of Fnd0ParamReqmentRevision object. In this note text user can type any text, and
    parameter, and its value pair in below format
     [<param1 name>: value1, value2,..]
    Example [Temperature:0,10,20,30]
    :var templateText: body_text property of Fnd0ParamReqmentRevision object.
    """
    clientId: str = ''
    inputDetails: StdNoteInput = None
    parametersOnRelation: List[GetStdNoteParamAndValues] = ()
    noteText: str = ''
    templateText: str = ''


@dataclass
class OpenStdNoteResponse(TcBaseObj):
    """
    'OpenStdNoteResponse' structure represents list of 'OpenStdNoteContents' structure along with the ServiceData.
    
    :var openNoteObjectsDetails: A list of 'GetStdNoteParamAndValues' structure that hold the information of
    Fnd0ParamReqmentRevision objects parameter values set in the context, note text, body_text property of
    Fnd0ParamReqmentRevision, and input structure as 'StdNoteInput'.
    :var sreviceData: The Service Data.
    """
    openNoteObjectsDetails: List[OpenStdNoteContents] = ()
    sreviceData: ServiceData = None


@dataclass
class SetStdNoteDetails(TcBaseObj):
    """
    'SetStdNoteDetails' structure represents Standard Note/Parametric Requirement details with note relation object,
    and values to set on Standard note relation.
    
    :var inputNoteDetails: A 'StdNoteInput' structure that hold the information of Standard note object and Standard
    note relation.
    :var values: List of Values to be set on Standard note.
    """
    inputNoteDetails: StdNoteInput = None
    values: List[SetStdNoteParameters] = ()


@dataclass
class SetStdNoteParameters(TcBaseObj):
    """
    'SetStdNoteParameters' structure represents Parameter and its value to be set on Standard Note Relation object.
    
    :var parameter: Name of parameter.
    :var value: Value to be set on parameter from the list of values.
    """
    parameter: str = ''
    value: str = ''


@dataclass
class SetStdNoteResponse(TcBaseObj):
    """
    'SetStdNoteResponse' structure represents list of 'SetStdNoteResult' structure containing Standard Note details
    along with the ServiceData.
    
    :var resultObjects: A list of 'SetStdNoteResult' structure that hold the information of Standard Note objects.
    :var serviceData: The Service Data.
    """
    resultObjects: List[SetStdNoteResult] = ()
    serviceData: ServiceData = None


@dataclass
class SetStdNoteResult(TcBaseObj):
    """
    'SetStdNoteResult' structure defines the output for each note when set Note SOA operation is invoked. It returns
    the new note text value set on parametreized Requirement.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var standardNote: Standard Note/Parametric Requirement Revision object.
    :var noteText: Value of new Note Text on Standard Note/Parametric Requirement Revision object
    """
    clientId: str = ''
    standardNote: Fnd0ParamReqmentRevision = None
    noteText: str = ''


@dataclass
class StdNoteInput(TcBaseObj):
    """
    'StdNoteInput' structure represents the input parameters of setting Standard note values. It includes Standard
    Note/Parametric Requirement Revision object and the object of 'Fnd0ListsParamReqments' relation by which it is
    attached to any Item/ ItemRevision.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var standardNote: Tag of Fnd0ParamReqmentRevision object.
    :var stdNoteRelation: Relation object of type Fnd0ListParamRequirements by which Fnd0ParamReqmentRevision is
    attached to selected object revision.
    """
    clientId: str = ''
    standardNote: Fnd0ParamReqmentRevision = None
    stdNoteRelation: Fnd0ListsParamReqments = None



from __future__ import annotations

from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FullTranslationStatus(TcBaseObj):
    """
    Fully defines a translation status: its associated enumeration, display name and description.
    
    :var status: TranslationStatus associated with the status
    :var statusName: Display name of the status
    :var statusDescription: Description of the status
    """
    status: TranslationStatus = None
    statusName: str = ''
    statusDescription: str = ''


@dataclass
class Language(TcBaseObj):
    """
    Contains information about a language
    
    :var languageCode: The name of the desired locale. The valid locale name should be in the format as outlined in the
    Java Standard Language (i.e. en_US for English, United States).
    :var languageName: The localized language name
    """
    languageCode: str = ''
    languageName: str = ''


@dataclass
class LanguageResponse(TcBaseObj):
    """
    Information about the list of languages
    
    :var languageList: An ordered list of languages
    :var serviceData: Any partial errors that may occur when filling this request
    """
    languageList: List[Language] = ()
    serviceData: ServiceData = None


@dataclass
class TranslationStatusResponse(TcBaseObj):
    """
    Response associated to some LanguageInformation operation calls
    
    :var fullTranslationStatuses: List of all the full translation statuses
    :var serviceData: The 'ServiceData'.
    """
    fullTranslationStatuses: List[FullTranslationStatus] = ()
    serviceData: ServiceData = None


class TranslationStatus(Enum):
    """
    Defines the status of a translation: master language, approved, pending, in-review and invalid.
    """
    Master = 'TranslationStatusMaster'
    Approved = 'TranslationStatusApproved'
    Pending = 'TranslationStatusPending'
    InReview = 'TranslationStatusInReview'
    Invalid = 'TranslationStatusInvalid'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import AllocationLine, AllocationWindow, AllocationMapRevision, BOMLine, RevisionRule, PSBOMView, AllocationMap, BOMWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AllocationContextInput(TcBaseObj):
    """
    The AllocationContextInput structure represents all the data necessary for creating an Allocation Context. The
    basic attributes that are required by ITK are passed as named elements in the structure.
    
    :var id: The ID of the AllocationMap object to be created. If empty, will be generated.
    :var name: The name of the AllocationMap object to be created. Optional input.
    :var type: The type of the AllocationMap object to be created. If type is not provided, the AllocationMap created
    will be of type AllocationMap.
    :var revision: The revision id for the AllocationRevisionMap object to be created.
    :var openedBOMWindows: List of BOMWindow business objects to be associated to the AllocationMap business object,
    where the created AllocationMap will be the context for the Allocations created between the BOMLine objects of
    these BOMWindow objects.
    """
    id: str = ''
    name: str = ''
    type: str = ''
    revision: str = ''
    openedBOMWindows: List[BOMWindow] = ()


@dataclass
class AllocationLineInfo(TcBaseObj):
    """
    The AllocationLineInfo structure represents all of the data necessary to construct the AllocationLine object. The
    basic attributes that are required by ITK are passed as named elements in the structure.
    
    :var name: The AllocationLine name for creation, optional if empty, generated on server.
    :var type: The AllocationLine type, if empty then the default type Allocation is used.
    :var reason: The AllocationLine reason, optional.
    :var fromBOMLines: AllocateFrom BOMLine objects, required.
    :var toBOMLines: AllocateTo BOMLine objects, required.
    """
    name: str = ''
    type: str = ''
    reason: str = ''
    fromBOMLines: List[BOMLine] = ()
    toBOMLines: List[BOMLine] = ()


@dataclass
class AllocationLineInput(TcBaseObj):
    """
    The AllocationLineInput structure contains all the elements necessary for the identifying an AllocationLine object,
    including AllocationLineInfo structure as well as an object reference to the AllocationLine itelf.
    
    :var allocationLine: AllocationLine object which needs to be modified.
    :var allocationLineInfo: AllocationLineInfo element containing all information necessary for modification of the
    AllocationLine object.
    """
    allocationLine: AllocationLine = None
    allocationLineInfo: AllocationLineInfo = None


@dataclass
class AllocationWindowInfo(TcBaseObj):
    """
    The AllocationWindowInfo structure represents all of the data necessary for opening an AllocationWindow. The basic
    attributes that are required by ITK are passed as named elements in the structure.
    
    :var allocationContext: AllocationContext Object, can be AllocationMap or AllocationMapRevision businessobject.
    :var allocationRule: Allocation Rule.
    :var openedBOMWindows: Keep track of opened BOM Window
    """
    allocationContext: AllocationMapRevision = None
    allocationRule: RevisionRule = None
    openedBOMWindows: List[BOMWindow] = ()


@dataclass
class GetAllocatedBOMViewInfo(TcBaseObj):
    """
    The GetAllocatedBOMViewInfo structure is used as a mapping between the AllocationMap
    object and its corresponding allocated BOMView Objects.
    
    :var allocationMap: AllocationMap business object.
    :var allocatedBOMViewObjects: A list of PSBOMView business objects associated to the AllocationMap object.
    """
    allocationMap: AllocationMap = None
    allocatedBOMViewObjects: List[PSBOMView] = ()


@dataclass
class GetAllocatedBOMViewResponse(TcBaseObj):
    """
    The GetAllocatedBOMViewResponse structure contains a list of the structure GetAllocatedBOMViewInfo and Service
    Data. Any errors are reported in ServiceData.
    
    :var allocatedBOMViewInfo: Structure which contains the AllocationMap business object and a list of associated
    PSBOMView objects.
    :var serviceData: The ServiceData.
    """
    allocatedBOMViewInfo: List[GetAllocatedBOMViewInfo] = ()
    serviceData: ServiceData = None


@dataclass
class GetAllocationWindowResponse(TcBaseObj):
    """
    The GetAllocationWindowResponse structure represents the current state of the BOMWindow as well as the list of
    AllocationLine objects contained in said BOMWindow.
    
    :var allocationWindow: The Business Object corresponding to the AllocationWindow.
    :var allocationLines: AllocationLine objects for the modified AllocationWindow.
    :var serviceData: ServiceData contains the created AllocationMap, AllocationMapRevision, and AllocationWindow 
    created in the created objects of list of the ServiceData Element. Any errors occurred during the operation will be
    returned in the Partial errors of the ServiceData element.
    """
    allocationWindow: AllocationWindow = None
    allocationLines: List[AllocationLine] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Core._2011_06.PropDescriptor import AttachedPropDescsResponse
from tcsoa.gen.BusinessObjects import BusinessObject, Envelope
from tcsoa.gen.Core._2011_06.Session import Credentials, LoginResponse, ClientCacheInfo
from tcsoa.gen.Core._2011_06.LOV import LOVAttachmentsResponse, LOVAttachmentsInput
from typing import List
from tcsoa.gen.Core._2011_06.Reservation import OkToCheckoutResponse
from tcsoa.gen.Common import PolicyType
from tcsoa.gen.Core._2007_06.PropDescriptor import PropDescInfo
from tcsoa.gen.Core._2011_06.OperationDescriptor import SaveAsDescResponse
from tcsoa.gen.Core._2011_06.DataManagement import TraceabilityInfoInput, ValidateRevIdsInfo, SaveAsIn, ValidateRevIdsResponse, SaveAsObjectsResponse, TraceabilityReportOutput
from tcsoa.gen.Server import ServiceData, TypeSchema
from tcsoa.utils import TcService


class OperationDescriptorService(TcService):

    @classmethod
    def getSaveAsDesc(cls, targetObjects: List[BusinessObject]) -> SaveAsDescResponse:
        """
        This operation returns information required to save a business object. The 'SaveAsDescriptor' includes the
        metadata information for the properties required when saving a business object, i.e., the property is
        mandatory, the property is visible, if value is to be copied from original object, etc.  The 'SaveAsDescriptor'
        also includes the 'DeepCopyData' which is a recursive data structure. The 'DeepCopyData' contains information
        about how the secondary objects (related and referenced objects) are to be copied (reference, copy as object,
        no copy).
        
        Use cases:
        Get SaveAsDescriptor for the SaveAs wizard
        Client constructs the SaveAs dialog for a Business Object using this operation. The information returned by
        this operation allows a client to construct the SaveAs dialogs and DeepCopy panels for user input. Once the
        user input is received, client can make subsequent invocation to the 'DataManagement.saveAsObjects' operation
        to execute SaveAs on the object.
        """
        return cls.execute_soa_method(
            method_name='getSaveAsDesc',
            library='Core',
            service_date='2011_06',
            service_name='OperationDescriptor',
            params={'targetObjects': targetObjects},
            response_cls=SaveAsDescResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def getTraceReport(cls, input: TraceabilityInfoInput) -> TraceabilityReportOutput:
        """
        This operation will generate a Trace Report on the objects selected by user.  User will get information about
        complying as well as defining objects which are connected to selected object using  FND_TraceLink or its
        subtype of GRM relation.
        
        Trace links can be between following objects:
        
                1.    Between occurrences of an ItemRevision
                2.    Between ItemRevisions
                3.    Between Items
                4.    Between any two WorkspaceObject.
        
        If indirect trace report flag is set to true during this operation, then user will get trace report for
        ItemRevision if selected object is Occurrence, and trace report for Items if selected objects is ItemRevision
        in addition to direct trace report for the selected object.
        
        
        Use cases:
        Suppose user created trace link between Requirement R1 as start point and R2 as end point and creates trace
        link from Requirement R3 as start and R1 as end point.
        When user runs traceability report on R1 requirement he will get R2 object as complying object and R3 will come
        as defining object.
        """
        return cls.execute_soa_method(
            method_name='getTraceReport',
            library='Core',
            service_date='2011_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=TraceabilityReportOutput,
        )

    @classmethod
    def saveAsObjects(cls, saveAsIn: List[SaveAsIn]) -> SaveAsObjectsResponse:
        """
        This operation is generic operation for saving of Business Objects. It will also save any secondary objects
        that also need to be saved based on deep copy rule information
        
        Use cases:
        Client constructs the SaveAs dialog for a Business Object using 'OperationDescriptor.getSaveAsDesc' operation. 
        The information returned by that operation allows client to construct the SaveAs dialogs and DeepCopy panels
        for user input. Once the user input is received, client can make subsequent invocation to this
        ('DataManagement.saveAsObjects') operation to execute SaveAs on the object.
        """
        return cls.execute_soa_method(
            method_name='saveAsObjects',
            library='Core',
            service_date='2011_06',
            service_name='DataManagement',
            params={'saveAsIn': saveAsIn},
            response_cls=SaveAsObjectsResponse,
        )

    @classmethod
    def validateRevIds(cls, inputs: List[ValidateRevIdsInfo]) -> ValidateRevIdsResponse:
        """
        Validates and/or modifies the Revision Id based on the naming rules/revision naming rules and user exit
        callbacks.
        
        Use cases:
        This operation is generally used to validate revision id before providing it as input for create, revise and
        save-as operations.
        """
        return cls.execute_soa_method(
            method_name='validateRevIds',
            library='Core',
            service_date='2011_06',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=ValidateRevIdsResponse,
        )


class SessionService(TcService):

    @classmethod
    def getTypeDescriptions(cls, typeNames: List[str]) -> TypeSchema:
        """
        Get the Meta data for the named Business Model object types. This includes type and property descriptions and
        LOV information.
        """
        return cls.execute_soa_method(
            method_name='getTypeDescriptions',
            library='Core',
            service_date='2011_06',
            service_name='Session',
            params={'typeNames': typeNames},
            response_cls=TypeSchema,
        )

    @classmethod
    def login(cls, credentials: Credentials) -> LoginResponse:
        """
        Authenticates the user's credentials and initialize a Teamcenter session for this client. The operation will
        throw an 'InvalidCredentialsException' if the 'username', 'password' or 'group' is not valid.
        When the client application is deployed to a 4Tier environment (communication through HTTP or TCCS) the login
        operation also contributes to the assignment of a Teamcenter server instance to the client session. The
        Teamcenter architecture varies from other client server architectures in that there is a dedicated instance of
        the Teamcenter server per client application. However, there are use cases where it is desirable for a single
        user to have multiple desktop applications running and each sharing a single instance of a Teamcenter server.
        This is controlled through the following elements:
        - hostPath    From the Connection class constructor, this specifies  the address (URI) the Teamcenter server is
        hosted on.
        - username    From this login operation, this specifies the user's Teamcenter user name.
        - sessionDiscriminator    From this login operation, this identifies the client session.
        
        
        
        The 'hostPath' argument determines the server machine that the client connects to. Once there, the pool manager
        on that host uses the 'username' and 'sessionDiscriminator' arguments of the login request to determine which
        Teamcenter server instance to assign the client to. If the pool manager has an existing Teamcenter server
        instance with the 'username'/'sessionDiscriminator' key, the client is assigned to that existing instance of
        the Teamcenter server, and therefore sharing the server with another client; otherwise, a new instance of the
        Teamcenter server is used. There are a few general scenarios for the 'sessionDiscriminator' argument:
        
        - Blank     If the user jdoe logs on to Teamcenter using two or more client applications using a blank
        'sessionDiscriminator' argument (for example, jdoe/ ), all of those clients are assigned to the same Teamcenter
        server instance. These client applications can be running on the same or different client hosts.
        - Constant     If the user jdoe logs on to Teamcenter using two or more client applications using a constant or
        fixed 'sessionDiscriminator' argument (for example, jdoe/MyApp ), those clients are assigned to the same
        Teamcenter server instance. This is similar to the blank 'sessionDiscriminator' argument; the difference is
        that only multiple instances of the client application using myApp started by jdoe share the same Teamcenter
        server instance.
        - Unique     If the user jdoe logs on using a unique random-generated string (for example, jdoe/akdk938lakc),
        the client application will be assigned to a dedicated instance of the Teamcenter server.
        
        
         
        The scenario you use depends on how your client application is used in the integrated environment. The most
        common case is the unique 'sessionDiscriminator' value.
        
        Exceptions:
        >When the credentials supplied are invalid or the requested locale is not allowed.  
        - 515143:     The logon was refused due to invalid 'username' or 'password'.
        - 515144:     The logon was refused due to invalid 'username' or 'password'.
        - 515142:     The logon was refused due to an invalid 'group'.
        - 128001: The logon was refused due to conflict with the encoding of the database instance. 
        - 128002:    The logon was refused due to missing localization.
        
        """
        return cls.execute_soa_method(
            method_name='login',
            library='Core',
            service_date='2011_06',
            service_name='Session',
            params={'credentials': credentials},
            response_cls=LoginResponse,
        )

    @classmethod
    def loginSSO(cls, credentials: Credentials) -> LoginResponse:
        """
        Authenticates the user using Single-Sign-On (SSO) credentials and initialize a Teamcenter session for this
        client. The 'username' and 'password' arguments must be obtained from Teamcenter's Security Services. The
        'SsoCredentials' class offers a simple API to get these values. The operation will throw an
        'InvalidCredentialsException' if the 'user', 'password' or 'group' is not valid.
        When the client application is deployed to a 4Tier environment (communication through HTTP or TCCS) the login
        operation also contributes to the assignment of a Teamcenter server instance to the client session. The
        Teamcenter architecture varies from other client server architectures in that there is a dedicated instance of
        the Teamcenter server per client application. However, there are use cases where it is desirable for a single
        user to have multiple desktop applications running and each sharing a single instance of a Teamcenter server.
        This is controlled through the following elements:
        - 'hostPath'    From the Connection class constructor, this specifies  the address (URI) the Teamcenter server
        is hosted on.
        - 'username'    From this login operation, this specifies the user's Teamcenter user name.
        - 'sessionDiscriminator'    From this login operation, this identifies the client session.
        
        
        
        The 'hostPath' argument determines the server machine that the client connects to. Once there, the pool manager
        on that host uses the 'username' and 'sessionDiscriminator' arguments of the logon request to determine which
        Teamcenter server instance to assign the client to. If the pool manager has an existing Teamcenter server
        instance with the 'username'/'sessionDiscriminator' key, the client is assigned to that existing instance of
        the Teamcenter server, and therefore sharing the server with another client; otherwise, a new instance of the
        Teamcenter server is used. There are a few general scenarios for the sessionDiscriminator argument:
        
        - Blank     If the user jdoe logs on to Teamcenter using two or more client applications using a blank
        'sessionDiscriminator' argument (for example, jdoe/ ), all of those clients are assigned to the same Teamcenter
        server instance. These client applications can be running on the same or different client hosts.
        - Constant     If the user jdoe logs on to Teamcenter using two or more client applications using a constant or
        fixed 'sessionDiscriminator' argument (for example, jdoe/MyApp ), those clients are assigned to the same
        Teamcenter server instance. This is similar to the blank 'sessionDiscriminator' argument; the difference is
        that only multiple instances of the client application using myApp started by jdoe share the same Teamcenter
        server instance.
        - Unique     If the user jdoe logs on using a unique random-generated string (for example, jdoe/akdk938lakc),
        the client application will be assigned to a dedicated instance of the Teamcenter server.
        
        
         
        The scenario you use depends on how your client application is used in the integrated environment. The most
        common case is the unique 'sessionDiscriminator' value.
        
        Exceptions:
        >When the credentials supplied are invalid or the requested locale is not allowed.  
        - 515143:    The logon was refused due to invalid 'username' or 'password'
        - 515144:    The logon was refused due to invalid 'username' or 'password'
        - 515142:    The logon was refused due to an invalid 'group'.
        - 128001:     The logon was refused due to conflict with the encoding of the database instance. 
        - 128002:    The logon was refused due to missing localization.
        
        """
        return cls.execute_soa_method(
            method_name='loginSSO',
            library='Core',
            service_date='2011_06',
            service_name='Session',
            params={'credentials': credentials},
            response_cls=LoginResponse,
        )

    @classmethod
    def updateObjectPropertyPolicy(cls, policyID: str, addProperties: List[PolicyType], removeProperties: List[PolicyType]) -> str:
        """
        Update the named policy, adding and removing the named properties. This operation only applies to object
        property policies that have been defined on the client side.
        
        Exceptions:
        >If the named policy does not exist in the server's memory (error code 214106).
        """
        return cls.execute_soa_method(
            method_name='updateObjectPropertyPolicy',
            library='Core',
            service_date='2011_06',
            service_name='Session',
            params={'policyID': policyID, 'addProperties': addProperties, 'removeProperties': removeProperties},
            response_cls=str,
        )

    @classmethod
    def getClientCacheData(cls, features: List[str]) -> ClientCacheInfo:
        """
        This operation returns information required to maintain a client cache. The Teamcneter server maintains a set
        of Datasets with static or near static data that is pertainant to a client application.  This static data can
        be downloaded through FMS to the cleint host one time, and available for each subsequent client session, thus
        improving the overall performance of the client application. These Datasets are updated as part of the deploy
        process from the BMIDE. The cleint cache consits of serveral features, each of these features can be used
        independnatly of each other. The following features are available:
        
        - ClientMetaModel :The is the client side version of the server`s Meta Model. This includes type descriptions,
        property descriptions, List Of Values data, and Dataset tool data. The use of the ClientMetaModel cache
        replaces the need to use the getTypeDescriptions  service calls. By setting the
        'Connection.setOption(OPT_USE_CLIENT_META_MODEL_CACHE, true)', the SOA client framework will use the cache for
        Client  Meta Model data. The SOA client framework takes care of calling this service opeation and FMS to
        populate the cache. This feature includes the Dataset:Types, Lov, ToolsData, types_local (one for each locale
        i.e types_en_US), lov_local (one for each locale i.e lov_en_US).
        - TextData: This contains the localized string available in the Teamcenter server's Text Server. Using the
        localized data from this cache replaces the need to call the getDisplayStrings service operation.
        
        """
        return cls.execute_soa_method(
            method_name='getClientCacheData',
            library='Core',
            service_date='2011_06',
            service_name='Session',
            params={'features': features},
            response_cls=ClientCacheInfo,
        )


class ReservationService(TcService):

    @classmethod
    def okToCheckout(cls, objects: List[BusinessObject]) -> OkToCheckoutResponse:
        """
        This operation determines whether or not the input objects may be checked out given the type of object, access
        rules, and current checkout state of the object.
        """
        return cls.execute_soa_method(
            method_name='okToCheckout',
            library='Core',
            service_date='2011_06',
            service_name='Reservation',
            params={'objects': objects},
            response_cls=OkToCheckoutResponse,
        )


class EnvelopeService(TcService):

    @classmethod
    def sendAndDeleteEnvelopes(cls, envelopes: List[Envelope]) -> ServiceData:
        """
        This operation sends mails to the recipients of each Envelope business object in envelopes. All the envelopes
        passed to this operation are deleted after they are processed, even if the processing is not successful. Each
        Envelope business object contains mail information such as subject, body, recipients, and attachments.
        Recipients can be Teamcenter users, groups and address lists, or, external email addresses. Teamcenter users
        receive envelopes in their Teamcenter Mailbox and also as emails if Mail_OSMail_activated site preference is
        set to 'true'.To send the emails, the site preference Mail_server_name should be properly configured. Any
        errors that occur while sending envelopes are returned as partial errors in ServiceData.
        
        Use cases:
        Use case 1: Send envelopes to Teamcenter users
        You can send envelopes to the Mailbox of Teamcenter users by specifying the users as recipients on Envelope
        business objects.  Also, email messages can be sent to Teamcenter users if Mail_OSMail_activated site
        preference is set to 'true'.
        
        User case 2: Send emails to external email addresses
        Email messages can be sent to external users by specifying their email addresses as recipients on Envelope
        business objects.
        """
        return cls.execute_soa_method(
            method_name='sendAndDeleteEnvelopes',
            library='Core',
            service_date='2011_06',
            service_name='Envelope',
            params={'envelopes': envelopes},
            response_cls=ServiceData,
        )


class PropDescriptorService(TcService):

    @classmethod
    def getAttachedPropDescs2(cls, inputs: List[PropDescInfo]) -> AttachedPropDescsResponse:
        """
        Returns a list of Property Descriptors based on the input structure.  The Property Descriptors contain the
        Display Name, Description and other information about the input property.
        
        Use cases:
        This operation provides following use case for retrieving a set of Property Descriptors given a type name and
        list of property names for that type.
        
        Use Case 1:Retrieve a set of Property Descriptors for a list of property names.
        - Create a new PropDescInfo input structure.
        - Populate the type name and input list of property names.
        - Call getAttachedPropDescs2 with the newly created input structure.
        
        
        
        Exceptions:
        >.
        """
        return cls.execute_soa_method(
            method_name='getAttachedPropDescs2',
            library='Core',
            service_date='2011_06',
            service_name='PropDescriptor',
            params={'inputs': inputs},
            response_cls=AttachedPropDescsResponse,
        )


class LOVService(TcService):

    @classmethod
    def getLOVAttachments(cls, objectStructArray: List[LOVAttachmentsInput]) -> LOVAttachmentsResponse:
        """
        Returns valid LOV attachments for the properties of each object passed in as input. It may return LOV or null
        based on condition validations. If none of the conditions evaluated to be True, then no LOV attachment is
        returned for the property of an instance.
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='getLOVAttachments',
            library='Core',
            service_date='2011_06',
            service_name='LOV',
            params={'objectStructArray': objectStructArray},
            response_cls=LOVAttachmentsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, BOMLine
from tcsoa.gen.Manufacturing._2012_02.DataManagement import GeneralInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetConnectorResponse(TcBaseObj):
    """
    Response for getConnectorInfo operation.
    
    :var connectorInfo: List of structures containing connector information for given ItemRevision objects.
    :var serviceData: Partial errors as part of the serviceData.
    """
    connectorInfo: List[ConnectorTableInfo] = ()
    serviceData: ServiceData = None


@dataclass
class GetPhysicalAttachmentsInput(TcBaseObj):
    """
    Input structure containing context and scope information for which the attachments need to be retrieved.
    
    :var context: A root Mfg0BvrWorkarea object in Bill of Equipment structure. The operation processes scope within
    context.
    :var scope: A Mfg0BvrWorkarea object under context for which attachment need to be queried
    """
    context: BOMLine = None
    scope: BOMLine = None


@dataclass
class GetPhysicalAttachmentsResponse(TcBaseObj):
    """
    Response for getPhysicalAttachmentsInScope operation.
    
    :var attachmentsInfo: List of structures containing information about the physical attachment relation
    Mfg0MEPhysicalAttachment or Mfg0MEMountToolToRobot between AbsOccurrence of BOMLine objects under the given scope
    Mfg0BvrWorkarea.
    :var serviceData: Partial errors as part of the serviceData.
    """
    attachmentsInfo: List[PhysicalAttachmentInfo] = ()
    serviceData: ServiceData = None


@dataclass
class PhysicalAttachmentInfo(TcBaseObj):
    """
    The structure containing source and target BOMLine and the Mfg0MEPhysicalAttachment relation information.
    
    :var context: A root Mfg0BvrWorkarea object in Bill of Equipment structure.
    :var scope: A Mfg0BvrWorkarea object under context
    :var source: A BOMLine node of the AbsOccurrence represeting primary  in MfgMEPhysicalAttachment or
    Mfg0MEMountToolToRobot relation.
    :var target: A BOMLine node of the AbsOccurrence represeting secondary  in MfgMEPhysicalAttachment or
    Mfg0MEMountToolToRobot relation.
    :var relationName: Name of relation by which  AbsOccurence of source BOMLine and target BOMLine are related.
    Possible values are Mfg0MEPhysicalAttachment or its subtype Mfg0MEMountToolToRobot.
    :var relationInfo: Structure containing Mfg0MEPhysicalAttachment or its subtype Mfg0MEMountToolToRobot relation
    properties.
    Teamcenter::Soa::Manufacturing::_2012_02::GeneralInfo is an existing Data Structure that holds map for different
    properties. 
    The relationInfo contains information about following
    - String type relation properties of Mfg0MEPhysicalAttachment. stringProps element holds these proerties as key.
    Below is the list of String type relation properties of Mfg0MEPhysicalAttachment
    - mfg0SourceID as key. Value can be  "Connect Point1".
    - mfg0SourceIDFormatType as key. Possible values are Kinematics (KIN), Connector Table (CTB), PRT, JT.
    - mfg0TargetID as key. Value can be "Mount Point1".
    - mfg0TargetIDFormatType as key. Possible values are Kinematics (KIN), Connector Table (CTB), PRT, JT.
    - Boolean type mfg0BiDirectional relation property of Mfg0MEPhysicalAttachment. boolProps element of GeneralInfo
    contains mfg0BiDirectional as key and value is true or false.
    - Double type mfg0RelTransform relation property of Mfg0MEPhysicalAttachment. doubleProps element of GeneralInfo
    contains mfg0RelTransform as key and value is double e.g. 100.00.
    - Double type mfg0TCPFPosition relation property of Mfg0MEMountToolToRobot. doubleProps element of GeneralInfo
    contains mfg0TCPFPosition as key and value is double e.g. 100.00. This is specified only if the relation type is
    Mfg0MEMountToolToRobot.
    
    """
    context: BOMLine = None
    scope: BOMLine = None
    source: BOMLine = None
    target: BOMLine = None
    relationName: str = ''
    relationInfo: GeneralInfo = None


@dataclass
class RemovePhysicalAttachmentRelInput(TcBaseObj):
    """
    Input structure containing context, scope, source, and target information.
    
    :var context: A root Mfg0BvrWorkarea object in Bill of Equipement structure. The operation deletes relations of
    type Mfg0MEPhysicalAttachment or Mfg0MEMountToolToRobot created in this context.
    :var scope: A Mfg0BvrWorkarea object under context.  The operation travereses the scope, finds the source and
    target BOMLine,gets  AbsOccurrence for source and target BOMLine and collects Mfg0MEPhysicalAttachment or
    Mfg0MEMountToolToRobot relation to be deleted.
    :var source: A BOMLine node of the AbsOccurrence represeting primary  in MfgMEPhysicalAttachment or
    Mfg0MEMountToolToRobot relation.
    :var target: A BOMLine node of the AbsOccurrence represeting secondary in MfgMEPhysicalAttachment or
    Mfg0MEMountToolToRobot relation.
    """
    context: BOMLine = None
    scope: BOMLine = None
    source: BOMLine = None
    target: BOMLine = None


@dataclass
class SetConnectorInput(TcBaseObj):
    """
    Input structure containing ItemRevision and connectorInformation
    
    :var itemRev: ItemRevision for which connector details are to be updated.
    :var connectorInformation: List of structures containing properties of connectors to be added or updated in
    Mfg0MEConnectorTableRow on Mfg0MEConnectorTable. 
    Teamcenter::Soa::Manufacturing::_2012_02::GeneralInfo is an existing Data Structure that holds map for different
    properties. 
    The connectorsInfo contains information about following
    - String type relation properties of Mfg0MEConnectorTableRow. stringProps element holds these proerties as key.
    Below is the list of String type relation properties of Mfg0MEConnectorTableRow
    - mfg0ConnectorType as key. Value can be  Tool Frame Connector, Base frame connector, Riser frame connector.
    - mfg0ConnectorID as key. Value can be "Mount Point1". 
    - mfg0ConnectorName as key. Value can be "Base Frame".
    - Double type mfg0RelToOrigin property of Mfg0MEConnectorTableRow. doubleProps element of GeneralInfo contains
    mfg0RelToOrigin as key and value is double e.g. 100.00.
    
    """
    itemRev: BusinessObject = None
    connectorInformation: List[GeneralInfo] = ()


@dataclass
class SetPhysicalAttachmentsInput(TcBaseObj):
    """
    Input structure containing context, scope, source, target, relation name, and relation properties information.
    
    :var context: A root Mfg0BvrWorkarea object in Bill of Equipment structure. In context of this root, AbsOccurrence
    is created for source BOMLine and target BOMLine.
    :var scope: A Mfg0BvrWorkarea object under context.
    :var source: A BOMLine node of the AbsOccurrence representing primary  in MfgMEPhysicalAttachment or
    Mfg0MEMountToolToRobot relation.
    :var target: A BOMLine node of the AbsOccurrence representing secondary in MfgMEPhysicalAttachment or
    Mfg0MEMountToolToRobot relation.
    :var relationName: Name of relation by which AbsOccurence of source BOMLine and target BOMLine are related.
    Possible values are  Mfg0MEPhysicalAttachment or Mfg0MEMountToolToRobot.
    :var relationInfo: Structure containing Mfg0MEPhysicalAttachment or Mfg0MEMountToolToRobot relation properties.
    Teamcenter::Soa::Manufacturing::_2012_02::GeneralInfo is an existing Data Structure that holds map for different
    properties.
    The relationInfo contains information about following
    - String type relation properties of Mfg0MEPhysicalAttachment. stringProps element holds these proerties as key.
    Below is the list of String type relation properties of Mfg0MEPhysicalAttachment
    - mfg0SourceID as key. Value can be  "Connect Point1"
    - mfg0SourceIDFormatType as key. Possible values are Kinematics (KIN), Connector Table (CTB), PRT, JT
    - mfg0TargetID as key. Value can be "Mount Point1"
    - mfg0TargetIDFormatType as key. Possible values are Kinematics (KIN), Connector Table (CTB), PRT, JT
    - Boolean type mfg0BiDirectional relation property of Mfg0MEPhysicalAttachment. boolProps element of GeneralInfo
    contains mfg0BiDirectional as key and value is true or false.
    - Double type mfg0RelTransform relation property of Mfg0MEPhysicalAttachment. doubleProps element of GeneralInfo
    contains mfg0RelTransform as key and value is double e.g. 100.00.
    - Double type mfg0TCPFPosition relation property of Mfg0MEMountToolToRobot. doubleProps element of GeneralInfo
    contains mfg0TCPFPosition as key and value is double e.g. 100.00. This is specified only if the relation type is
    Mfg0MEMountToolToRobot.
    
    """
    context: BOMLine = None
    scope: BOMLine = None
    source: BOMLine = None
    target: BOMLine = None
    relationName: str = ''
    relationInfo: GeneralInfo = None


@dataclass
class ConnectorTableInfo(TcBaseObj):
    """
    The structure containing ItemRevision and its connector details.
    
    :var itemRev: ItemRevision for which connector details are to be retrieved.
    :var connectorProperties: List of structure containing Mfg0MEConnectorTableRow properties.
    Teamcenter::Soa::Manufacturing::_2012_02::GeneralInfo is an existing Data Structure that holds map for different
    properties. 
    The connectorsInfo contains information about following
    - String type relation properties of Mfg0MEConnectorTableRow. stringProps element holds these proerties as key.
    Below is the list of String type relation properties of Mfg0MEConnectorTableRow
    - mfg0ConnectorType as key. Value can be  Tool Frame Connector, Base frame connector, Riser frame connector.
    - mfg0ConnectorID as key. Value can be "Mount Point1". 
    - mfg0ConnectorName as key. Value can be "Base Frame"
    - Double type mfg0RelToOrigin property of Mfg0MEConnectorTableRow. doubleProps element of GeneralInfo contains
    mfg0RelToOrigin as key and value is double e.g. 100.00.
    
    """
    itemRev: BusinessObject = None
    connectorProperties: List[GeneralInfo] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Fnd0Message, Awb0Element, Awb0ProductContextInfo
from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_05.Compare import CompareResultsCursor
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class CompareContentData2(TcBaseObj):
    """
    Input containing Awb0Element objects, configuration information and depth describing how deep traversal and compare
    to be performed.
    
    :var source: 'CompareContentInfo'2 for source object to compare.
    :var target: 'CompareContentInfo'2 for target object to compare.
    :var startFreshCompare: If true, a fresh compare is initiated.
    :var compareInBackground: If true, compare will be initiated in background.
    :var notificationMessage: Notification message containing the compare results.
    :var sourceCursor: 'CompareResultsCursor' for source.
    :var targetCursor: 'CompareResultsCursor' for target.
    :var compareOptions: A map (string, list of strings) containing an option for compare as key and list of its
    associated attributes as values. The following supported key/values are:
    
    Key "FilteringRule" with following list of values.
    - AccountabilityAll - Includes all lines.
    - AccountabilityLeavesOnly - Includes only leaf nodes; excludes all hierarchy nodes.
    - AccountabilityLinkedAssmOrLeaves - Includes assigned assembly nodes but excludes their children. If an assembly
    is not assigned, includes its leaf nodes.
    - AccountabilityMOA - To compare source and destination BOMs, traversing or skipping occurrences in the source
    structure based on the occurrence type of the linked occurrences in the destination structure.
    - DesignToBomAll - To validate completeness between part BOM and CAD BOM.
    
    
    
    Key "DisplayOptions" with following list of valid values.
    - PartialMatch &ndash; Compare specific set of properties for a match between source and target structure.
    - FullMatch &ndash; Compare for exact match between source and target structure.
    
    """
    source: CompareContentInfo2 = None
    target: CompareContentInfo2 = None
    startFreshCompare: bool = False
    compareInBackground: bool = False
    notificationMessage: Fnd0Message = None
    sourceCursor: CompareResultsCursor = None
    targetCursor: CompareResultsCursor = None
    compareOptions: CompareOptionsMap = None


@dataclass
class CompareContentInfo2(TcBaseObj):
    """
    Input structure for storing source and target information for content comparison.
    
    :var element: Awb0Element whose children are to be compared.
    :var productContextInfo: Awb0ProductContextInfo containing configuration information for the element.
    :var visibleUids: A list of unique ID for element for which compare differences values to be returned.
    :var depth: Level to which the source or target is to be expanded for compare.
    Value greater than 0 means level to which the source or target is to be expanded.
    Value -1 means all levels of structure will be compared.
    Value 0 means only leaf nodes to be compared.
    Value -2 means it is expected to get saved results if exists any.
    :var filteringRule: A list of filtering rules to be used for compare. This can contain AccountabilityAll ,
    AccountabilityLeavesOnly, AccountabilityLinkedAssmOrLeaves etc.
    """
    element: Awb0Element = None
    productContextInfo: Awb0ProductContextInfo = None
    visibleUids: List[str] = ()
    depth: int = 0
    filteringRule: str = ''


@dataclass
class CompareOptionsResponse(TcBaseObj):
    """
    Contains response of operation getCompareOptions.
    
    :var compareOptions: A map (string, list of strings) containing an option for compare as key and list of its
    associated attributes as values. The following supported key/values are:
    
    Key "FilteringRule" with following list of values.
    - AccountabilityAll - Includes all lines.
    - AccountabilityLeavesOnly - Includes only leaf nodes; excludes all hierarchy nodes.
    - AccountabilityLinkedAssmOrLeaves - Includes assigned assembly nodes but excludes their children. If an assembly
    is not assigned, includes its leaf nodes.
    - AccountabilityMOA - To compare source and destination BOMs, traversing or skipping occurrences in the source
    structure based on the occurrence type of the linked occurrences in the destination structure.
    - DesignToBomAll - To validate completeness between part BOM and CAD BOM.
    
    
    
    Key "DisplayOptions" with following list of valid values.
    - PartialMatch &ndash; Compare specific set of properties for a match between source and target structure.
    - FullMatch &ndash; Compare for exact match between source and target structure.
    
    """
    compareOptions: CompareOptionsMap = None


@dataclass
class CompareResp2(TcBaseObj):
    """
    Structure to store response of content comparison.
    
    :var sourceDifferences: A map( string, int ) of unique identifer and status for source. The map comprises of unique
    identifier of Awb0Element and its integer status in the source structure as compared to the target structure.UID of
    Awb0Element is followed by list of unique identifiers (UID) of equivalent Awb0Elements if any. The Awb0Element
    object can have one or more equivalent objects identified using equivalence and partial match criteria.The
    equivalent objects&rsquo; unique identifiers are populated when diff has status 2.
    Integer value indicates below:
    2 (Modified) &ndash; The object represented by the unique identifier has one or more property value differences
    when compared to target.
    3 (Missing from Target) - The object represented by the unique identifier is missing in target.
    :var targetDifferences: A map( string, int ) of unique identifer and status for target. The map comprises of unique
    identifier of Awb0Element and its integer status in the target structure as compared to the source structure.UID of
    Awb0Element is followed by list of unique identifiers (UID) of equivalent Awb0Elements if any. The Awb0Element
    object can have one or more equivalent objects identified using equivalence and partial match criteria.The
    equivalent objects&rsquo; unique identifiers are populated when diff has status 2.
    Integer value indicates below:
    1 (Missing from Source) &ndash; The object represented by the unique identifier is missing from source.
    2 (Modified) &ndash; The object represented by the unique identifier has one or more property value differences
    when compared to source.
    :var sourceDepth: Level to which comparison of source structure is done.
    Value greater than 1 means the level till which structure was compared.Value -1 means all levels of structures were
    compared.Value 0 means only leaf nodes were compared.
    :var totalNoOfSourceDifferences: Number of total source differences found for the given input.
    :var totalNoOfTargetDifferences: Number of total target differences found for the given input.
    :var serviceData: The service data.
    :var pagedSourceDifferences: Compare results data for source.
    :var pagedTargetDifferences: Compare results data for target.
    :var sourceCursor: 'CompareResultsCursor' for source.
    :var targetCursor: 'CompareResultsCursor' for target.
    :var timestampOfStoredResults: Time stamp of stored compare results if found any for the given compare input
    options.
    :var compareOptions: A map (string, list of strings) containing an option for compare as key and list of its
    associated attributes as values. The following supported key/values are:
    
    Key "FilteringRule" with following list of values.
    - AccountabilityAll - Includes all lines.
    - AccountabilityLeavesOnly - Includes only leaf nodes; excludes all hierarchy nodes.
    - AccountabilityLinkedAssmOrLeaves - Includes assigned assembly nodes but excludes their children. If an assembly
    is not assigned, includes its leaf nodes.
    - AccountabilityMOA - To compare source and destination BOMs, traversing or skipping occurrences in the source
    structure based on the occurrence type of the linked occurrences in the destination structure.
    - DesignToBomAll - To validate completeness between part BOM and CAD BOM.
    
    
    
    Key "DisplayOptions" with following list of valid values.
    - PartialMatch &ndash; Compare specific set of properties for a match between source and target structure.
    - FullMatch &ndash; Compare for exact match between source and target structure.
    
    
    :var targetDepth: Level to which comparison of target structure is done.
    Value greater than 1 means the level till which structure was compared.Value -1 means all levels of structures were
    compared.Value 0 means only leaf nodes were compared.
    """
    sourceDifferences: CompareContentOutputMap2 = None
    targetDifferences: CompareContentOutputMap2 = None
    sourceDepth: int = 0
    totalNoOfSourceDifferences: int = 0
    totalNoOfTargetDifferences: int = 0
    serviceData: ServiceData = None
    pagedSourceDifferences: List[CompareResultsRespData2] = ()
    pagedTargetDifferences: List[CompareResultsRespData2] = ()
    sourceCursor: CompareResultsCursor = None
    targetCursor: CompareResultsCursor = None
    timestampOfStoredResults: datetime = None
    compareOptions: CompareOptionsMap = None
    targetDepth: int = 0


@dataclass
class CompareResultsRespData2(TcBaseObj):
    """
    Output structure for compare results.
    
    :var uids: Unique identifier (UID) of Awb0Element followed by list of unique identifiers (UID) of equivalent
    Awb0Elements if any. The Awb0Element object can have one or more equivalent objects identified using equivalence
    and partial match criteria.The equivalent objects&rsquo; unique identifiers are populated when diff has status 2.
    :var diff: Status of Awb0Element in comparison results. Integer value indicates below:
    1 (Missing from Source) &ndash; The object represented by the unique identifier is missing from source.
    2 (Modified) &ndash; The object represented by the unique identifier has one or more property value differences
    when compared to source. 
    3 (Missing from Target) - The object represented by the unique identifier is missing in target.
    """
    uids: str = ''
    diff: int = 0


"""
Map( string , int ) to store output from content comparision.
"""
CompareContentOutputMap2 = Dict[str, int]


"""
A map (string, list of strings) containing an option for compare as key and list of its associated attributes as values.
"""
CompareOptionsMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from typing import List
from tcsoa.gen.Internal.Rdv._2009_01.VariantManagement import MetaExprTokens
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class NVEMetaExprTokensResponse(TcBaseObj):
    """
    This structure contains list of 'MetaExprTokens' and the 'ServiceData' object.
    
    :var metaExprs: Contains the list of 'MetaExprTokens' objects.
    :var serviceData: The ServiceData object containing full/partial errors.
    """
    metaExprs: List[MetaExprTokens] = ()
    serviceData: ServiceData = None


@dataclass
class SearchResponse(TcBaseObj):
    """
    'SearchResponse' structure contains the location of the plmxml file on the transient volume, in which the search
    results are available.
    
    :var plmxmlFileLocation: The pruned plmxml file location to download to client
    :var serviceData: The 'ServiceData' object containing the PartialError details, if any.
    """
    plmxmlFileLocation: str = ''
    serviceData: ServiceData = None


@dataclass
class SearchResults(TcBaseObj):
    """
    'SearchResults' structure contains all the search results along with errors if any, after the search completion
    
    :var backgroundBOMLines: Vector of BOMLine objects that matched the search criteria
    :var unconfiguredBOMLines: Vector of BOMLine objects that did not match the search criteria but are still part of
    the product context
    :var unconfigurableBOMLines: Vector of BOMLine objects that were encountered during the process of matching the
    bookmark strings for which no revision configures using the current RevisionRule, leaving some bookmark strings
    unmatched
    :var unconfiguredVOOBOMLines: Vector of BOMLine objects that got unconfigured by applying the Valid Overlay Options
    with the input criteria
    :var serviceData: An object of 'ServiceData' which contains any errors if occurred during execution of the VOO
    operation.
    """
    backgroundBOMLines: List[BOMLine] = ()
    unconfiguredBOMLines: List[BOMLine] = ()
    unconfigurableBOMLines: List[BOMLine] = ()
    unconfiguredVOOBOMLines: List[BOMLine] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.StructureManagement._2018_11.Structure import CreateGroupInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class StructureService(TcService):

    @classmethod
    def createInterchangeableGroups(cls, inputs: List[CreateGroupInput]) -> ServiceData:
        """
        Creates Occurrence level substitute groups or Item/ItemRevision level (global) alternate groups based on the
        input object type. 
        
        Interchangeable Groups are groups of "stockable parts" that needs to be all replaced together as repairs are
        made. They can be defined at two levels:
        &bull;    Occurrence: List of occurrences that must be replaced together with a list of Item objects within the
        same BVR.
        &bull;    Item/Item Revision (Global): List of Item/ItemRevision objects that must be replaced together with
        another list of Item objects &ndash; regardless of the assembly where they are used.
        
        Use cases:
        This operation creates an interchangeable group based on the input object type, source objects, interchangeable
        parts list and any properties to set on the interchangeable group. The new interchangeable group will be
        returned through 'ServiceData'.
        &bull;    User wants to provide a substitute group for a list of components in a structure. So in the input the
        user can choose Substitute Group type, provide source occurrences and select a list of Item object as
        substitute parts. The new substitute group for the structure will be returned through 'ServiceData'.
        &bull;    User wants to create a substitute group but also identify additional properties with properties
        input. So in the input the user can choose Substitute Group type, provide source occurrences and substitute
        parts list, also input additional properties. The new substitute group will be returned through 'ServiceData'.
        &bull;    User wants to create an alternate group. So in the input the user sets Alternate Group type, provide
        source Item/ItemRevision objects, and select a list of Item objects as alternate parts. The new alternate group
        will be returned through 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='createInterchangeableGroups',
            library='StructureManagement',
            service_date='2018_11',
            service_name='Structure',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanFile, Awb0ProductContextInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class SaveVisBookmarkInfoResponse(TcBaseObj):
    """
    This structure contains a list of Awb0ProductContextInfo instances,  ImanFile object and the read file ticket to
    the ImanFile object  that contains the visualization bookmark information that was saved.
    
    :var visBookmarkInfos: A list of 'VisBookmarkInfo' structures.
    :var serviceData: The Service Data through which the errors are communicated to the client.
    """
    visBookmarkInfos: List[VisBookmarkInfo] = ()
    serviceData: ServiceData = None


@dataclass
class SaveVisBookmarkInput(TcBaseObj):
    """
    This structure contains  the  Awb0ProductContextInfo, and transient file ticket for the bookmark file that was
    uploaded to the transient volume.
    
    :var productContextInfo: Awb0ProductContextInfo for which the visualization bookmark information is updated.
    :var visBookmarkTransientFileTicket: FMS transient file ticket for a file which was uploaded to the transient
    volume.
    """
    productContextInfo: Awb0ProductContextInfo = None
    visBookmarkTransientFileTicket: str = ''


@dataclass
class VisBookmarkInfo(TcBaseObj):
    """
    This structure contains  the  Awb0ProductContextInfo, and its visualization bookmark information. The visualization
    bookmark information is contained in the file for which the FMS read ticket is being returned.
    
    :var productContextInfo: Awb0ProductContextInfo for which the visualization bookmark information was saved.
    :var visBookmarkFile: ImanFile object containing the visualization bookmark information that was saved.
    :var visBookmarkFileReadTicket: FMS read ticket for the ImanFile object.
    """
    productContextInfo: Awb0ProductContextInfo = None
    visBookmarkFile: ImanFile = None
    visBookmarkFileReadTicket: str = ''


@dataclass
class GetVisBookmarkInfoResponse(TcBaseObj):
    """
    This structure contains a list of Awb0ProductContextInfo instances,  ImanFile object and the read file ticket to
    the ImanFile object  that contains the visualization bookmark information.
    
    :var visBookmarkInfos: A list of 'VisBookmarkInfo' structures.
    :var serviceData: The Service Data through which the errors are communicated to the client.
    """
    visBookmarkInfos: List[VisBookmarkInfo] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Allocations._2011_06.Allocation import AllocationContextInput2
from tcsoa.gen.Allocations._2007_01.Allocation import GetAllocationWindowResponse
from tcsoa.utils import TcService


class AllocationService(TcService):

    @classmethod
    def createAllocationContext2(cls, input: AllocationContextInput2) -> GetAllocationWindowResponse:
        """
        The operation creates an AllocationMap object for the given name, id and attribute map input. This operation
        has Multi field key support for AllocationMap business object creation. The created AllocationMap object is
        saved to Teamcenter. It creates an AllocationWindow with the AllocationMapRevision object as context. It adds
        the input BOMWindow objects as the BOMWindow objects for the AllocationWindow. The created AllocationMap,
        AllocationRevision, AllocationWindow are returned as created objects list in ServiceData Element.
        
        Use cases:
        Create AllocationMap object with Multi field key support
        The AllocationMap object can be created with full Multi field key support using this operation. If the business
        constant of MFK for the AllocationMap object has item_id and any other attribute, then the user can create
        AllocationMap with same item id as well. 
        """
        return cls.execute_soa_method(
            method_name='createAllocationContext2',
            library='Allocations',
            service_date='2011_06',
            service_name='Allocation',
            params={'input': input},
            response_cls=GetAllocationWindowResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Query._2007_09.SavedQuery import SavedQueriesResponse, QueryInput
from tcsoa.utils import TcService


class SavedQueryService(TcService):

    @classmethod
    def executeSavedQueries(cls, input: List[QueryInput]) -> SavedQueriesResponse:
        """
        Executes a set of saved queries of following types: QRY_RUN_BY_TC ( 0 ), QRY_RUN_BY_KEYWORD_SEARCH ( 24 ). 
         
         The saved queries can be executed to yield results in 2 modes: 
        (1) Flat mode: In this traditional execution mode, only the first-level objects (corresponding to the queried
        class) satisfying the query are returned
        (2) Hierarchical/Indented mode: This mode is only applicable for saved queries that allow Hierarchical/Indented
        results. In this execution mode, the first-level objects as well as any sub-level objects satisfying the query
        criteria are returned. The hierarchy level information is also returned so that result subsets can be
        reconstructed using the resulting objects. 
         
        This service will retun the matched object UIDs. After fetching UIDs, client need to call
        DataManagementService.loadObjects operation to load objects by pages. 
        
         See the QueryInput and QueryResults data structures for usage details.
        """
        return cls.execute_soa_method(
            method_name='executeSavedQueries',
            library='Query',
            service_date='2007_09',
            service_name='SavedQuery',
            params={'input': input},
            response_cls=SavedQueriesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2017_11.ResourceManagement import AddMultiToolCutterResponse, DeleteMultiToolCutterResponse
from tcsoa.gen.Internal.Manufacturing._2017_11.StructureManagement import PasteByRuleResponse, AlternativeScopeForProductInputInfo, PasteByRuleInfoInput, AlternativeScopeForProductResponse
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def pasteByRule(cls, inputInfo: PasteByRuleInfoInput) -> PasteByRuleResponse:
        """
        This operation copies the Mfg0BvrOperation lines from a Bill of Process (BOP) structure and paste  them to a
        PlantBOP. The paste behavior is controlled via a set of preferences. This operation accepts as input a list of
        BOMLine objects of type Mfg0BvrOperation or Mfg0BvrProcess.  The input BOMLine objects are traversed using the
        closure rule specified  in the preference PasteByRuleClosureRule to collect Mfg0BvrOperation lines.  For each
        Mfg0BvrOperation a new line will be created and pasted as a child of the target Mfg0BvrStation in the PlantBOP
        . An error will be issued if the same MEOperation already exist on any MEStation. Attachments and the child
        BOMLine objects of the source Mfg0BvrOperation line  are handled according to the cloning rule specified by the
        preference PasteByRuleCloningRule.
        
        The new BOMline creation and paste behaviour are controlled via the following set of preferences:
        - MEPasteByRuleOccProperties &ndash; list of occurrence properties to align between original source line and
        new equivalent line under the target line.
        - MEPasteByRuleClosureRule &ndash; name of closure rule to use to filter the input source lines.
        - MEPasteByRuleCloningRule &ndash; the cloning rule to use when creating new target line from source line and
        attachments.
        - MEPasteByRuleDatasetWarning &ndash; a list of dataset types if present on the source line that will raise a
        warning and will not be carried over.
        
        
        
        Use cases:
        Paste Operation/Process from BOP to MEStation in PlantBOP:
        
        Let&rsquo;s assume the source and target structures as below: 
        
        BOP (Source Structure)
           Process
              Operation1 (source)
                 ConsumedLine
                 Attached Dataset
              Process1 (source)
                 Operation2
                    Attached Dataset
           
        
        PlantBOP (Target Structure)
           ProcessArea
              ProcessStation (target)
        
                                  The user sends Operation1 and Process1 as the source lines and the Process Station as
        the 
                                  target line as parameters to this operation.  The resulting structure will look like
        below:
                                  
                   PlantBOP
                                     ProcessArea
                                        ProcessStation
                 Operation1 (New)
                    ConsumedLine
                    Attached Dataset (Referenced)
              
                 Operation2 (New)
                    Attached Dataset (Referenced)
        
        New Operation1 and Operation2 lines are created and added to Process Station. Additionally, references to the
        original operations attachments are made.
        """
        return cls.execute_soa_method(
            method_name='pasteByRule',
            library='Internal-Manufacturing',
            service_date='2017_11',
            service_name='StructureManagement',
            params={'inputInfo': inputInfo},
            response_cls=PasteByRuleResponse,
        )

    @classmethod
    def createAlternativeScopeForProduct(cls, altenativeScpForProdInfo: List[AlternativeScopeForProductInputInfo]) -> AlternativeScopeForProductResponse:
        """
        This service operation creates new Fnd0MfgAlternativeScope (a sub-class of AppearanceGroup) for each input root
        node of Bill of Material (BOM) structure and establishes a relation between them.
        Additionally, this operation creates child Fnd0MfgAlternativeScope objects below the newly created
        Fnd0MfgAlternativeScope. These child objects act as containers for the information specified in the input and
        stores the input BOMLine objects under them.
        
        Use cases:
        Use Case 1 : User opens a MECollaborationContext (CC) object, select some of the scope from Bill Of Process
        (BOP) structure to create Alternative CC. On the structure information page, user selects "Partial" scope for
        the BOM structure. A new section for the "Alternative Scope" is displayed. User can also select BOMLines from
        the BOM structure to add in the "Fnd0MfgAlternativeScope".
        An Fnd0MfgAlternativeScope containing  sub scopes with the selected obejcts from BOM structure is created.
        """
        return cls.execute_soa_method(
            method_name='createAlternativeScopeForProduct',
            library='Internal-Manufacturing',
            service_date='2017_11',
            service_name='StructureManagement',
            params={'altenativeScpForProdInfo': altenativeScpForProdInfo},
            response_cls=AlternativeScopeForProductResponse,
        )


class ResourceManagementService(TcService):

    @classmethod
    def addMultiToolCutter(cls, strItemRevUID: str) -> AddMultiToolCutterResponse:
        """
        This operation creates a new multitool cutter for an existing tool item revision. The new cutter is an
        additional Classification object (ICO) that is connected to the item revision (multiple classification of an
        item revision). 
        The operation classifies the ItemRevision specified by input parameter "strItemRevUID" into the same class in
        which the ItemRevision is already classified. 
        UID for newly created ICO and cutter ID for the same will be returned in the response from this operation. This
        operation will not create an ICO if the ItemRevision revision is not classified.
        This operation will internally check if the existing tool already has multitool classification objects attached
        and calculate the next cutter ID. If the tool has no existing multitool cutters defined, the default cutter ID
        is "2" (The existing ICO is the first cutter.). Otherwise, the next available cutter number is calculated. The
        new value of the cutter ID will be written into the "Cutter ID" attribute (Attribute ID: -45041) of the newly
        created ICO. Also the shared "Multitool" attribute (Attribute ID: - 45040) value will be set to "Yes" during
        execution of this service operation.
        The attributes "Cutter ID" and "Multitool" will be available on tool assembly classes as part of the MRL 4.1
        kit.
        
        Use cases:
        Multitools are tools those have more than one cutter. For example: a multitool with two cutters could support
        turning for one edge and grooving for the other. As another example, a multitool could have three cutters all
        supporting the same turning operation type. This operation will be useful when user wants to add a cutter to an
        existing tool item revision which is classified.
        """
        return cls.execute_soa_method(
            method_name='addMultiToolCutter',
            library='Internal-Manufacturing',
            service_date='2017_11',
            service_name='ResourceManagement',
            params={'strItemRevUID': strItemRevUID},
            response_cls=AddMultiToolCutterResponse,
        )

    @classmethod
    def deleteMultiToolCutter(cls, deleteICOUID: str) -> DeleteMultiToolCutterResponse:
        """
        This operation deletes multitool cutter with given UID (specified in the input parameter deleteICOUID) for an
        ItemRevision. The multitool cutter is a  Classification object (ICO) that is connected to the ItemRevision
        (multiple classification of an ItemRevision). After a multitool cutter is deleted and the number of cutters was
        reduced to one, the shared "Multitool" attribute (Attribute ID: -45040)  value will be set to empty. The tool
        assembly is then no longer a multitool. This will imply that there is only one Classification object (ICO)
        connected to the ItemRevision.
        
        The attribute "Multitool" will be available on tool assembly classes as part of the MRL 4.1 kit.
        
        Use cases:
        Multitools are tools those have more than one cutter. For example: a multitool with two cutters could support
        turning for one edge and grooving for the other. As another example, a multitool could have three cutters all
        supporting the same turning operation type.
        This operation will be useful when user wants to delete a cutter from an existing tool ItemRevision which is
        classified.
        """
        return cls.execute_soa_method(
            method_name='deleteMultiToolCutter',
            library='Internal-Manufacturing',
            service_date='2017_11',
            service_name='ResourceManagement',
            params={'deleteICOUID': deleteICOUID},
            response_cls=DeleteMultiToolCutterResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.VendorManagementAW._2020_05.VendorManagement import AssignPartnerContractInput2, RemovePartnerContractInput2
from tcsoa.gen.BusinessObjects import ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService
from tcsoa.gen.Internal.VendorManagementAW._2019_12.VendorManagement import ViewModelRowsResponse


class VendorManagementService(TcService):

    @classmethod
    def assignPartnerContractToObject2(cls, input: AssignPartnerContractInput2) -> ServiceData:
        """
        This operation assigns the partner contract to the given WorkspaceObject object as specified in the input. The
        selected partner contract will be assigned to the given context information.
        """
        return cls.execute_soa_method(
            method_name='assignPartnerContractToObject2',
            library='Internal-VendorManagementAW',
            service_date='2020_05',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def getAssignedProductsOfContract(cls, input: ItemRevision) -> ViewModelRowsResponse:
        """
        This operation returns the information of the assigned products of a given partner contract revision.
        """
        return cls.execute_soa_method(
            method_name='getAssignedProductsOfContract',
            library='Internal-VendorManagementAW',
            service_date='2020_05',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=ViewModelRowsResponse,
        )

    @classmethod
    def removePartnerContractFromObject2(cls, input: List[RemovePartnerContractInput2]) -> ServiceData:
        """
        This operation removes the partner contract from the given WorkspaceObject object as specified in the input.
        The partner contract will beis removed from the structure configuration of selected object. The selected object
        could be a ConfigurationContext or unconfigured ItemRevision.
        """
        return cls.execute_soa_method(
            method_name='removePartnerContractFromObject2',
            library='Internal-VendorManagementAW',
            service_date='2020_05',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Cad._2007_01.StructureManagement import AssemblyArrangementInfo, AbsOccInfo, RelativeStructureChildInfo
from tcsoa.gen.BusinessObjects import AssemblyArrangement, ItemRevision, BOMLine
from tcsoa.gen.Cad._2007_06.StructureManagement import ClassicOptionInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.Cad._2007_09.StructureManagement import VariantCondInfo
from dataclasses import dataclass
from datetime import datetime


@dataclass
class BomLineVariantCondition(TcBaseObj):
    """
    This contains the variant condition information for a given BOMLine object.
    
    :var bomLine: Refers to BOMLine object on which variant condition are defined.
    :var conditionClauses: Refers to a list of VariantCondInfo struct objects which has classic variant condition
    information.
    """
    bomLine: BOMLine = None
    conditionClauses: List[VariantCondInfo] = ()


@dataclass
class VariantConditionResponse(TcBaseObj):
    """
    This contains the variant condition information for a set of BOMLine objects.
    
    :var variantConditions: Refers to a list of 'BomLineVariantCondition' struct objects, contains the variant
    condition on BOMLine object.
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information.
    """
    variantConditions: List[BomLineVariantCondition] = ()
    serviceData: ServiceData = None


@dataclass
class ClassicOptionData(TcBaseObj):
    """
    Contains the option information for a single item revision.
    
    :var itemRevision: itemRevision
    :var optionData: optionData
    """
    itemRevision: ItemRevision = None
    optionData: List[ClassicOptionInfo] = ()


@dataclass
class ClassicOptionsResponse(TcBaseObj):
    """
    Contains the option information for a set of ItemRevision objects.
    
    :var itemRevisionOptionData: Refers to a list of 'ClassicOptionData' struct, which has ItemRevision and
    corresponding classic variant option list.
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the service
    operation, plain objects, and error information.
    """
    itemRevisionOptionData: List[ClassicOptionData] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateAbsoluteStructureInfo2(TcBaseObj):
    """
    Contains Last Modified Date of BVR, contextItemRev, List of AbsOccInfo for bvr qualified overrides and a list of
    AssemblyArrangementInfo for bvr/arrangement qualified overrides.
    
    :var lastModifiedOfBVR: Last Modified Date of BVR
    :var contextItemRev: ItemRevision object reference of the context assembly to create/validate the occurrence,
    required reference
    :var bvrAbsOccInfo: List of AbsOccInfo for bvr qualified overrides
    :var arrAbsOccInfo: List of AssemblyArrangementInfo for bvr/arrangement qualified overrides, may be null
    """
    lastModifiedOfBVR: datetime = None
    contextItemRev: ItemRevision = None
    bvrAbsOccInfo: List[AbsOccInfo] = ()
    arrAbsOccInfo: List[AssemblyArrangementInfo] = ()


@dataclass
class CreateOrUpdateAbsoluteStructurePref2(TcBaseObj):
    """
    Contains a flag to check whether BVR needs to be modified, if input last modified date is different from actual and
    a cadOccIdAttrName which identifies the BOMLine attribute that is used to identify relative occurrences to update.
    
    :var overwriteForLastModDate: Flag to check whether BVR needs to be modified, if input last modified date is
    different from actual.
    :var cadOccIdAttrName: Identifies the BOMLine attribute that is used to identify relative occurrences to update.
    """
    overwriteForLastModDate: bool = False
    cadOccIdAttrName: str = ''


@dataclass
class CreateOrUpdateRelativeStructureInfo2(TcBaseObj):
    """
    Contains lastModifiedOfBVR, a parent ItemRevision object, list of type RelativeStructureChildInfo and a  boolean
    value to convey precision of the BVR.
    
    :var lastModifiedOfBVR: Last Modified Date of BVR
    :var parent: ItemRevision object reference for which the context assembly is created or updated, required reference
    :var childInfo: List of type RelativeStructureChildInfo
    :var precise: Flag for updating the BVR to precise(true)/imprecise(false)
    """
    lastModifiedOfBVR: datetime = None
    parent: ItemRevision = None
    childInfo: List[RelativeStructureChildInfo] = ()
    precise: bool = False


@dataclass
class CreateOrUpdateRelativeStructurePref2(TcBaseObj):
    """
    Contains overwriteForLastModDate, cadOccIdAttrName and a list of item types.
    
    :var overwriteForLastModDate: Flag to check whether BVR needs to be modified, if input last modified date is
    different from actual.
    :var cadOccIdAttrName: String representing the occurrence note type which holds the value for the CAD occurrence id
    or PSOccurrenceThread uid
    :var itemTypes: List of item types that the client is interested in, such that if the overall structure in
    Teamcenter contains structure relating to other item types or subtypes not in this list,
    that structure will not be deleted if this operation is complete.
    """
    overwriteForLastModDate: bool = False
    cadOccIdAttrName: str = ''
    itemTypes: List[str] = ()


@dataclass
class DeleteAssemblyArrangementsInfo2(TcBaseObj):
    """
    Information to delete assembly arrangements.
    
    :var lastModifiedOfBVR: Last modified date of BOM view revision object under the input 'itemRev'.  This input is
    not required.  If this input date is different than the current last modified date and the
    'overwriteForLastModDate' preference is false the input will be ignored and processing will continue with the next
    input.  In this scenario, error 215033 will be returned.  If the dates are different and the
    'overwriteForLastModDate' preference is true, processing will continue with the current input, the BVR will be
    modified and the arrangements deleted.  In this scenario, error 215034 will be returned.
    :var itemRev: Object reference of the item revision context assembly from which the assembly arrangements are to be
    removed.  This is a required input.  An error will be returned if a valid 'itemRev' is not specified.
    :var arrangements: List of assembly arrangement object references to be deleted.
    """
    lastModifiedOfBVR: datetime = None
    itemRev: ItemRevision = None
    arrangements: List[AssemblyArrangement] = ()


@dataclass
class DeleteAssemblyArrangementsPref(TcBaseObj):
    """
    Preference structure for 'deleteAssemblyArrangements'.
    
    :var overwriteForLastModDate: Flag to check whether the structure needs to be updated if the input last modified
    date is different from the current last modified date of the object in Teamcenter.  If false, but the
    'DeleteAssemblyArrangementsInfo2' 'lastModifiedOfBVR' input specified is different than the set modified date,
    partial error 215033 will be returned.
    """
    overwriteForLastModDate: bool = False


@dataclass
class DeleteRelativeStructureInfo2(TcBaseObj):
    """
    Contains lastModifiedOfBVR, parent itemRevision and childInfo.
    
    :var lastModifiedOfBVR: Last Modified Date of BVR
    :var parent: ItemRevision object reference for the context assembly from which children are to be removed
    :var childInfo: List of identifiers of the relative occurrences to be deleted. This is the CAD occurrence id or
    PSOccurrenceThread uid to uniquely identify the occurrence under a particular context Item Revision.
    """
    lastModifiedOfBVR: datetime = None
    parent: ItemRevision = None
    childInfo: List[str] = ()


@dataclass
class DeleteRelativeStructurePref2(TcBaseObj):
    """
    Preference structure for 'deleteRelativeStructure'.
    
    :var overwriteForLastModDate: Flag to check whether the structure needs to be updated if the input last modified
    date is different from the current last modified date of the object in Teamcenter.  If false, but the
    'DeleteRelativeStructureInfo3' 'lastModifiedOfBVR' input specified is different than the set modified date, partial
    error 215033 will be returned.
    :var cadOccIdAttrName: The BOMLine attribute that contains the CAD occurrence identifier.
    """
    overwriteForLastModDate: bool = False
    cadOccIdAttrName: str = ''



from __future__ import annotations

from tcsoa.gen.StructureManagement._2013_05.VariantManagement import BOMVariantOptionValueEntry
from tcsoa.gen.BusinessObjects import BusinessObject, BOMWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ApplyBOMVariantRulesResponse2(TcBaseObj):
    """
    It contains list of variant rules applied to a BOM window.
    
    :var rules: A list of BOMVariantRuleContents2 object associated with the BOMWidow
    :var serviceData: Object of service data, that returns partial errors.
    :var window: BOMWindow object on which rules have been applied
    """
    rules: List[BOMVariantRuleContents2] = ()
    serviceData: ServiceData = None
    window: BOMWindow = None


@dataclass
class BOMVariantRuleContents2(TcBaseObj):
    """
    Contains variant rule, the saved variant rules associated with the window, list of option value details and a
    boolean flag for constraints evaluation status.
    
    In getBOMVariantRules2 operation, this structure used as output. 
    
    In setBOMVariantRule2 operation, this structure used as input as well as output. As input, it contains details of
    variant rule, saved variant rule, saved variant rule modification flag and list of options that needs to be set on
    variant rule. As output, it contains details of variant rule with set option values. 
    
    In applyBOMVariantRules2 operation, this structure used as input as well as output. Typically the variantRule, svr,
    isSVRModified provide input and list of BOMVariantOptionValueEntry and isConstraintsEvaluated are used for output. 
    
    Constraints (defaults, derived defaults and rule check) will only be evaluated for single variant rule having
    single option value.
    
    :var variantRule: VariantRule with which window has been configured. This cannot be NULL.
    :var svr: Saved variant rule with which window has been configured. This will be NULL if window is not configured
    with saved variant rule.
    :var isSVRModified: True, if saved variant rule has been modified.
    :var bomVariantOptionValueEntry: List of options associated with VariantRule.
    :var isConstraintsEvaluated: True, if default, derived default have been evaluated.
    """
    variantRule: BusinessObject = None
    svr: BusinessObject = None
    isSVRModified: bool = False
    bomVariantOptionValueEntry: List[BOMVariantOptionValueEntry] = ()
    isConstraintsEvaluated: bool = False


@dataclass
class BOMVariantRuleOutput2(TcBaseObj):
    """
    Contains window and the list of rule associated with window. It has the identifier to map input variant rule to
    output variant rule.
    
    :var window: The BOMWindow on which rules are set.
    :var rules: A list of BOMVariantRuleContents2 objects.
    :var clientId: Identifier to map input variant rules to output variant rules.
    """
    window: BOMWindow = None
    rules: List[BOMVariantRuleContents2] = ()
    clientId: str = ''


@dataclass
class BOMVariantRulesResponse2(TcBaseObj):
    """
    List of varaint rule data associated with window.
    
    :var variantRuleData: A list of BOMVariantRuleOutput2 object.
    :var serviceData: Object of service data.
    """
    variantRuleData: List[BOMVariantRuleOutput2] = ()
    serviceData: ServiceData = None


@dataclass
class SetBOMVariantRuleData2(TcBaseObj):
    """
    Contains information about window and variant rule that needs to be set on the window. This structure will be used
    as an input and as an output. Input contains details of window and BOMVariantRuleContents2 that needs to be set on
    window. Output contains details of BOMVariantRuleContents2 having information about option values set on the
    variant rule.
    
    :var window: A BOMWindow object on which variant rules are to be set.
    :var rules: A list of BOMVariantRuleContents2 object which contain details of Variant Rule and list of options and
    values.
    :var clientId: Identifier string to map input variant rule to output variant rule.
    """
    window: BOMWindow = None
    rules: List[BOMVariantRuleContents2] = ()
    clientId: str = ''


@dataclass
class SetBOMVariantRulesResponse2(TcBaseObj):
    """
    SetBOMVariantRulesResponse2 object reference.
    
    :var variantRuleData: A list of objects containing SetBOMVariantRuleData2 object.
    :var serviceData: Object of service data that that returns partial errors.
    """
    variantRuleData: List[SetBOMVariantRuleData2] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import WorkspaceObject, ItemRevision
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AssignPartnerContractInput2(TcBaseObj):
    """
    The structure holds information of the partner contract to assign, selected object, preferred status and
    configuration context.
    
    :var selectedObject: Selected ManufacturerPart or ItemRevision to assign partner contract.
    :var configurationContext: Awb0SavedBookmark or Fnd0AppSession object of selectedObject.
    :var partnerContract: Vm0PrtnrContractRevision object to assign.
    :var preferredStatus: Preferred status for partner contract in the context of an object. Valid values are given in
    LOV preference: "Vm0PrtnrContractAssignmentStatus".
    """
    selectedObject: WorkspaceObject = None
    configurationContext: WorkspaceObject = None
    partnerContract: ItemRevision = None
    preferredStatus: str = ''


@dataclass
class RemovePartnerContractInput2(TcBaseObj):
    """
    The structure holds information of the partner contract to remove and selected object.
    
    :var selectedObject: ManufacturerPart or ItemRevision and its subtypes.
    :var partnerContract: Vm0PrtnrContractRevision object to remove.
    :var configurationContext: Awb0SavedBookmark or Fnd0AppSession object of selectedObject.
    """
    selectedObject: WorkspaceObject = None
    partnerContract: ItemRevision = None
    configurationContext: WorkspaceObject = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class SrchConnectedLinesOutput(TcBaseObj):
    """
    Object containing input connection and connected BOMLines.
    
    :var connectionLine: Input connection BOMLine.
    :var connectedLines: Connected BOMLine objects.
    """
    connectionLine: BusinessObject = None
    connectedLines: List[BusinessObject] = ()


@dataclass
class SrchConnectedLinesResponse(TcBaseObj):
    """
    Return object for searchConnectedLines operation.
    
    :var output: List of objects containing input connection and connected BOMLines.
    :var serviceData: Standard service data.
    """
    output: List[SrchConnectedLinesOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Ai._2006_03.Ai import ApplicationRef
from tcsoa.gen.BusinessObjects import ConfigurationContext, RequestObject, BOMWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateScopedMultipleStructure2Response(TcBaseObj):
    """
    GenerateScopedMultipleStructure2Response struct
    
    :var ticket: The transient file ticket to be used for downloading the generated plmxml
    :var data: partial failures are returned - along with object ids for each plmxml data could not be generated.
    """
    ticket: str = ''
    data: ServiceData = None


@dataclass
class GenerateScopedSyncRequest2Response(TcBaseObj):
    """
    GenerateScopedSyncRequest2Response struct
    
    :var request: request
    :var data: partial failures are returned - object ids for each sync data could not be created.
    """
    request: RequestObject = None
    data: ServiceData = None


@dataclass
class ObjectsWithConfig(TcBaseObj):
    """
    structure to specify multiple objects with each set potentially having it's own configuration
    
    :var apprefs: vector of ApplicationRefs specifying the objects to be included in the plmxml generation
    :var config: configuration to be used for the above set of objects.
    :var refContexts: If passing process object, this parameter can be used to specify how to
    setup the reference Product/Plant structure for that Process. If unused,
    pass an empty vector.
    """
    apprefs: List[ApplicationRef] = ()
    config: Configuration = None
    refContexts: List[ReferenceContext] = ()


@dataclass
class ReferenceContext(TcBaseObj):
    """
    structure to specify the reference structure, in the case of process element like MEProcessRevision,
    MEOpRevision, a process APN etc.
    
    :var topLineObject: ApplicationRef specifying the object to be set as the topline of the reference structure.
    It is optional if config structure's configuringObject is an StructureContext object.
    If not needed - pass an empty ApplicationRef.
    :var config: configuration to be used to construct the reference window.
    """
    topLineObject: ApplicationRef = None
    config: Configuration = None


@dataclass
class CompareConfigurationContextsResponse(TcBaseObj):
    """
    CompareConfigurationContextsResponse struct
    
    :var compareResults: bool array pointing to the result in the compares. Only successful (true/false) are returned
    here.
    :var serviceData: any partial errors are reported here. For example: if any pair comparison fails -
    that failed index with the failed message will be reported like this.
    """
    compareResults: List[bool] = ()
    serviceData: ServiceData = None


@dataclass
class Configuration(TcBaseObj):
    """
    Configuration structure.
    
    :var existingWindow: BOMWindow representing the window from which target application was launched. Must be
    initialized to NULL by client if unused.
    :var useDefaultRevisionRule: If true use default RevisionRule. Used if none of the above options are provided.
    :var revRuleName: Name of RevisionRule. Used if existingWindow and configuringObject are not provided.
    :var variantRule: ApplicationReference representing classic VariantRule.Used in case existingWindow is not provided
    and other options do not yield a VariantRule.
    :var configuringObject: A CollaborationContext or StructureContext object.Used if existingWindow is not provided.
    """
    existingWindow: BOMWindow = None
    useDefaultRevisionRule: bool = False
    revRuleName: str = ''
    variantRule: ApplicationRef = None
    configuringObject: ApplicationRef = None


@dataclass
class ConfigurationContextPair(TcBaseObj):
    """
    ConfigurationContextPair struct
    
    :var src: src configurationcontext object to use in compare
    :var other: target configurationcontext object to use in compare.
    """
    src: ConfigurationContext = None
    other: ConfigurationContext = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0Element, Awb0ProductContextInfo
from tcsoa.gen.Internal.ActiveWorkspaceBom._2018_12.Compare import CompareOptionsMap
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CompareContentAsyncData2(TcBaseObj):
    """
    Input containing Awb0Element objects, configuration information and various actions using which compare is to be
    performed.
    
    :var source: 'CompareContentAsyncInfo' for source object to compare.
    :var target: 'CompareContentAsyncInfo' data for target object to compare.
    :var compareOptionsMap: A map (string, list of strings) containing an option for compare as key and list of its
    associated attributes as values.This can have equivalence criterias to use , list of partial match properties to
    use for comparison.
    """
    source: CompareContentAsyncInfo2 = None
    target: CompareContentAsyncInfo2 = None
    compareOptionsMap: CompareOptionsMap = None


@dataclass
class CompareContentAsyncInfo2(TcBaseObj):
    """
    Input containing Awb0Element objects, configuration information and depth describing how deep traversal and compare
    to be performed.
    
    :var element: Awb0Element whose children are to be compared.
    :var productContextInfo: Awb0ProductContextInfo containing configuration information for the element.
    :var depth: Level to which the source or target is to be expanded for compare.
    
    If depth > 0 it signifies level to which the source is target is to be expanded for compare.
    If depth == 0 then only leaf nodes will be compared.
    If depth == -1 then all levels of structure will be compared.
    """
    element: Awb0Element = None
    productContextInfo: Awb0ProductContextInfo = None
    depth: int = 0



from __future__ import annotations

from tcsoa.gen.ConfigFilterCriteria._2011_06.EffectivityManagement import ConfigExpression
from typing import List
from tcsoa.gen.BusinessObjects import POM_object
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class EffectivityConditionInfo(TcBaseObj):
    """
    This structure provides input data for the operation such as the list of the objects which are affected and the
    effectivity expressions to be processed.
    
    :var targetObjects: A list of all affected business objects on which effectivity is to be set. Any POM_object or
    subtype supporting effectivity behavior is the valid input for this parameter.
    :var expression: The effectivity expression that should be set on the affected objects. Effectivity is set using
    either ConfigFormula, nested ConfigExpressions or EffectivityTableRow in this structure
    """
    targetObjects: List[POM_object] = ()
    expression: ConfigExpression = None


@dataclass
class EffectivityConditionSource(TcBaseObj):
    """
    This structure contains sourceObject and sourceExpression for which the effectivity is to be obtained.
    
    :var sourceObject: The business object which should be used to extract the effectivity expression. Any POM_object
    or  subtype supporting effectivity behavior is the valid input for this parameter.
    :var sourceExpression: This is ConfigExpression contains effectivity expression that should be used as source
    effectivity expression to process. When both fields are populated, sourceObject will be used and sourceExpression
    will be ignored.
    """
    sourceObject: POM_object = None
    sourceExpression: ConfigExpression = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Requirementsmanagement._2010_09.RequirementsManagement import MoveLineResponse, MoveLineInfo
from tcsoa.utils import TcService


class RequirementsManagementService(TcService):

    @classmethod
    def moveLine(cls, input: List[MoveLineInfo]) -> MoveLineResponse:
        """
        This operation manipulates the BOMLine Structure (Requirement/ Function/ SEBlock structure) by moving the
        selected BOMLine object either Upwards, Downwards or to a specific position. A Number property will be updated
        for selected Fnd0BuildingBlockBOMLine object along with its children. 
        Following operation are supported by this service operation.
        - Move Up - To move up the selected BOM line (Fnd0BuildingBlockBOMLine) up in the structure with respect to its
        sibling. For instance, if a requirement structure "Req_01", we have two children "SubReq_01" and "SubReq_02".
        In move up structure modification can be done with respect to the parent. Requirements "SubReq_01" and
        "SubReq_02" can be move up in context of parent "Req_01".
        - Move Down - To move down the selected BOM line (Fnd0BuildingBlockBOMLine) up in the structure with respect to
        its sibling. For instance, if a requirement structure "Req_01", we have two children "SubReq_01" and
        "SubReq_02". In move down structure modification can be done with respect to the parent. Requirements
        "SubReq_01" and "SubReq_02" can be move down in context of parent "Req_01".
        - Promote - To indent the selected BOM line (Fnd0BuildingBlockBOMLine) up in the structure with respect to its
        sibling. For instance, promote a requirement previously occupying level 2, with number "1.1", moves to the
        level 1, with number "2.0". Children previously occupying level 3 with number "1.1.1", moves to the level 2,
        with number "2.1".
        - Demote - To out-dent the selected BOM line (Fnd0BuildingBlockBOMLine) up in the structure with respect to its
        sibling. For instance, demote a requirement previously occupying level 2, with number "1.0", moves to the level
        3, with number "1.1.1". Children previously occupying level 3, with number "1.1.1", moves to the level 4, with
        number "1.1.1.1".
        - Edit Number - To move the selected BOM line (Fnd0BuildingBlockBOMLine) up in the structure with respect to
        its sibling. You can able to move selected BOM line object from one level to other and from one parent to other
        parent.  For instance, if a requirement has number "1.1.1" which means it is under parent "1.1". If the number
        changed to "1.1.3", the requirement is repositioned under the parent. If the number is changed to "2.1", then
        requirement will be removed from the parent and put under the sibling of the parent.
        
        
        
        Use cases:
        You can manipulate the hierarchy for a selected Requirement /Function/ SEBlockBOMLine object by using the
        moveLine operation. The given Fnd0BuildingBlockBOMLine object will be moved as per the choice along with its
        children and the Number property will be updated with new values.
        """
        return cls.execute_soa_method(
            method_name='moveLine',
            library='Requirementsmanagement',
            service_date='2010_09',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=MoveLineResponse,
        )



from __future__ import annotations

from tcsoa.gen.ConfigFilterCriteria._2011_06.EffectivityManagement import ConfigExpression
from tcsoa.gen.ConfigFilterCriteria._2013_05.EffectivityManagement import EffectivityDisplayStringResponse, EffectivityOverlapStateResponse
from typing import List
from tcsoa.utils import TcService
from tcsoa.gen.ConfigFilterCriteria._2013_05.VariantManagement import VariantCriteriaResponse


class EffectivityManagementService(TcService):

    @classmethod
    def getEffectivityDisplayString(cls, expressions: List[ConfigExpression], productName: str, productNameSpace: str, configuratorURL: str) -> EffectivityDisplayStringResponse:
        """
        This operation returns a localized string representation for effectivity expressions. This string
        representation is the same that is used for display values of effectivity formula properties
        mdl0effectivity_formula and mdl0allowed_eff_formula. Applications can use this API to display effectivity
        conditions for objects that have not yet been saved with this effectivity, e.g. if multiple effectivity
        modifications are accumulated in a client session before they are saved. In this scenario effectivity
        properties are not (yet) available prior to saving modified objects.
        This operation connects to the effectivity configurator service whose service endpoint is specified by the
        'configuratorURL' parameter. The actual conversion to formula strings is performed by this configurator
        service. Depending on the (configurator specific) encoding the operation might require the specification of a
        product context using parameters 'productName' and 'productNameSpace'. For example, a given configurator might
        use  shorthand representations for formulae if these are unique in the context of the specified product.
        Teamcenter 10.1 only supports local built in Teamcenter configurators where parameter 'configuratorURL' can be
        set to an empty string. Teamcenter configurators don't require a product context if the formulae are in
        Explicit Teamcenter Language. The encoding is explicit if all lexemes are uniquely identified, e.g.
        [OptionNamespace]FamilyName = UniqueValue, where no product context is required to determine the family name
        for a value, or the option namespace for the family. A configuration condition formula is in Explicit
        Teamcenter Language if its form is explicit and comprised of the lexemes documented for the Teamcenter Variant
        Formula property. Teamcenter configurators support shorthand encodings like FamilyName = UniqueValue or
        UniqueValue if the lexemes used in the shorthand encoding are unique in the specified product context.
        
        Use cases:
        An application prepares several effectivity conditions with the intent to save them to a set of objects using
        operation setEffectivityConditions. The application wants to display the effectivity display string that would
        get saved with setEffectivityConditions before actually saving them.
        
        Exceptions:
        >This operation may raise 'Teamcenter::Soa::Server::ServiceException 'wrapping around following Teamcenter
        errors:
        
        256000    An internal configurator error occurred.
        256001    A general configurator error occurred.
        256002    The following product is invalid: [DictionaryNamespace]FamilyName (Context: info).
        256003    The following feature value is invalid: [DictionaryNamespace]FamilyName (Context: info).
        256004    The following feature family is invalid: [DictionaryNamespace]FamilyName (Context: info).
        256005    An invalid feature combination was found (Context: info).
        256006    An invalid product configuration was found (Context: info).
        256007    An invalid expression was found (Context: info).
        256008    The following Boolean operator is invalid: opcode (Context: info).
        256009    An invalid effectivity date was found (Context: info).
        256010    A time-out occurred (Context: info).
        256011    A resource is not available (Context: info).
        256012    The following functionality is not implemented: function (Version: TcVersion).
        256013    A configurator implementation specific error occurred (Context: info).
        256014    The effectivity expression contains unmatched quote at position number.
        256015    The effectivity expression contains unmatched bracket at position number.
        256016    An unexpected token 'text' was found at position number.
        256017    The entry 'text' was found at position number when an option value was expected.
        256018    The token 'text' is missing after 'text' at position number.
        256019    The effectivity expression adaptor does not support text used at position number.
        256020    The XML tag 'text' has no text value.
        256021    The string 'formula' cannot be converted into an expression. 
        256022    The operator code 'opcode' in token at position number is unknown.
        256023    The effectivity value 'text' in token at position number is unknown.
        256024    The effectivity family 'text' in token at position number is unknown.
        256025    An ambiguous option value name 'FamilyName' in namespace 'DictionaryNamespace' in token at position
        number was found: text.
        256026    An ambiguous option family name 'FamilyName' in token at position number was found: 'text'.
        256027    The option namespace 'DictionaryNamespace' in token at position number is invalid.
        256028    The effectivity family 'FamilyName' in token at position number is invalid.
        256029    The effectivity option value 'text' for family 'FamilyName' in token at position number is invalid.
        256030    The effectivity option value 'text' for family 'FamilyName' in token at position number is invalid:
        its value is not numeric.
        256031    The effectivity option value 'text' for family 'FamilyName' in token at position number is invalid:
        the value unit of measure 'text' does not match the family unit of measure 'text'.
        256032    The option / family / family namespace combination 'text' / 'FamilyName' / 'DictionaryNamespace' in
        token at position number is invalid.
        256033    The family 'FamilyName' for option / family namespace combination 'text' / 'DictionaryNamespace' in
        token at position number is invalid. 
        256034    There is no option value in token at position number.
        256035    The effectivity option value 'text' for family 'FamilyName' in token at position 'number' is invalid:
        the value is not a valid date.
        """
        return cls.execute_soa_method(
            method_name='getEffectivityDisplayString',
            library='ConfigFilterCriteria',
            service_date='2013_05',
            service_name='EffectivityManagement',
            params={'expressions': expressions, 'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL},
            response_cls=EffectivityDisplayStringResponse,
        )

    @classmethod
    def getEffectivityOverlapStates(cls, referenceExpressions: List[ConfigExpression], expressions: List[ConfigExpression], productName: str, productNameSpace: str, configuratorURL: str) -> EffectivityOverlapStateResponse:
        """
        This operation determines and returns the degree of overlap between a set of effectivity expressions and a set
        of reference effectivity criteria expressions. When applications create or update DesignElements (DE) and
        assign effectivity, the UI may want to qualify the degree of overlap between the effectivity of the DE and the
        currently active RevRule effectivity criteria.
        The following example illustrates the overlap states for a set of expressions:
        'RevisionRule:     |---------------|
        Object 0:  |----|                            : OverlapStateNone
        Object 1:     |------|                       : OverlapStateIntersect
        Object 2:                |----|              : OverlapStateSubset
        Object 3:                    |------|        : OverlapStateIntersect
        Object 4:  |-----------------------------|   : OverlapStateSuperset
        Object 5:         |---------------|          : OverlapStateMatch
        Object 6:         |-------------------|      : OverlapStateSuperset
        Object 7:     |-------------------|          : OverlapStateSuperset'
        This operation connects to the effectivity configurator service whose service endpoint is specified by the
        'configuratorURL' parameter. Any conversion from formula strings is performed by this configurator service.
        Depending on the (configurator specific) encoding the operation might require the specification of a product
        context using parameters 'productName' and 'productNameSpace'. For example, a given configurator might
        recognize shorthand representations for formulae if these are unique in the context of the specified product.
        Teamcenter 10.1 only supports local built in Teamcenter configurators where parameter 'configuratorURL' can be
        set to an empty string. Teamcenter configurators don't require a product context if the formulae are in
        Explicit Teamcenter Language. The encoding is explicit if all lexemes are uniquely identified, e.g.
        [OptionNamespace]FamilyName = UniqueValue, where no product context is required to determine the family name
        for a value, or the option namespace for the family. A configuration expression formula is in Explicit
        Teamcenter Language if its form is explicit and comprised of the lexemes documented for the Teamcenter Variant
        Formula property. Teamcenter configurators support shorthand encodings like FamilyName = UniqueValue or
        UniqueValue if the lexemes used in the shorthand encoding are unique in the specified product context.
        
        Use cases:
        An application wants to qualify effectivity conditions that were retrieved with getEffectivityConditions as to
        whether the condition is equal to, intersects with, or is a subset or superset of the effectivity criteria
        associated with one or more RevisionRules. This cannot be achieved with properties on effectivity conditions or
        RevisionRules because the result depends on the combination of an effectivity condition and the effectivity
        configuration criteria on a RevisionRule. One and the same condition may have different overlap states with
        different RevisionRules.
        The application calls getEffectivityOverlapStates and passes the effectivity criteria (as obtained from a
        RevisionRule using getRevRuleEffectivityCriteria) as 'referenceExpressions', and the effectivity conditions (as
        obtained from a set of product data elements using getEffectivityConditions) as 'expressions'.
        
        
        Exceptions:
        >This operation may raise 'Teamcenter::Soa::Server::ServiceException 'wrapping around following Teamcenter
        errors:
        
        256000    An internal configurator error occurred.
        256001    A general configurator error occurred.
        256002    The following product is invalid: [DictionaryNamespace]FamilyName (Context: info).
        256003    The following feature value is invalid: [DictionaryNamespace]FamilyName (Context: info).
        256004    The following feature family is invalid: [DictionaryNamespace]FamilyName (Context: info).
        256005    An invalid feature combination was found (Context: info).
        256006    An invalid product configuration was found (Context: info).
        256007    An invalid expression was found (Context: info).
        256008    The following Boolean operator is invalid: opcode (Context: info).
        256009    An invalid effectivity date was found (Context: info).
        256010    A time-out occurred (Context: info).
        256011    A resource is not available (Context: info).
        256012    The following functionality is not implemented: function (Version: TcVersion).
        256013    A configurator implementation specific error occurred (Context: info).
        256014    The effectivity expression contains unmatched quote at position number.
        256015    The effectivity expression contains unmatched bracket at position number.
        256016    An unexpected token 'text' was found at position number.
        256017    The entry 'text' was found at position number when an option value was expected.
        256018    The token 'text' is missing after 'text' at position number.
        256019    The effectivity expression adaptor does not support text used at position number.
        256020    The XML tag 'text' has no text value.
        256021    The string 'formula' cannot be converted into an expression. 
        256022    The operator code 'opcode' in token at position number is unknown.
        256023    The effectivity value 'text' in token at position number is unknown.
        256024    The effectivity family 'text' in token at position number is unknown.
        256025    An ambiguous option value name 'FamilyName' in namespace 'DictionaryNamespace' in token at position
        number was found: text.
        256026    An ambiguous option family name 'FamilyName' in token at position number was found: 'text'.
        256027    The option namespace 'DictionaryNamespace' in token at position number is invalid.
        256028    The effectivity family 'FamilyName' in token at position number is invalid.
        256029    The effectivity option value 'text' for family 'FamilyName' in token at position number is invalid.
        256030    The effectivity option value 'text' for family 'FamilyName' in token at position number is invalid:
        its value is not numeric.
        256031    The effectivity option value 'text' for family 'FamilyName' in token at position number is invalid:
        the value unit of measure 'text' does not match the family unit of measure 'te    xt'.
        256032    The option / family / family namespace combination 'text' / 'FamilyName' / 'DictionaryNamespace' in
        token at position number is invalid.
        256033    The family 'FamilyName' for option / family namespace combination 'text' / 'DictionaryNamespace' in
        token at position number is invalid. 
        256034    There is no option value in token at position number.
        256035    The effectivity option value 'text' for family 'FamilyName' in token at position 'number' is invalid:
        the value is not a valid date.
        """
        return cls.execute_soa_method(
            method_name='getEffectivityOverlapStates',
            library='ConfigFilterCriteria',
            service_date='2013_05',
            service_name='EffectivityManagement',
            params={'referenceExpressions': referenceExpressions, 'expressions': expressions, 'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL},
            response_cls=EffectivityOverlapStateResponse,
        )


class VariantManagementService(TcService):

    @classmethod
    def getProductConfigurations(cls, productName: str, productNameSpace: str, configuratorURL: str) -> VariantCriteriaResponse:
        """
        This operation returns a set of product configurations managed at the configurator level, say for product
        tracking or reporting purposes.
        Teamcenter variant configurators manage product configurations as VariantRules which are attached to an
        ItemRevision representing the product using any relationship type (see also preference
        'TC_Default_SVR_Relationship'). The product is identified by 'productName' and 'productNameSpace' parameters.
        Teamcenter configurators use a Multiple Field Key (MFK) stable identifier (see property'
        Item::fnd0VariantNamespace') of the product item for 'productName', and the revision ID for 'productNameSpace'.
        The identifiers of the product associated with a Collaborative Design (Cpd0CollaborativeDesign) can be obtained
        from properties 'Mdl0ApplicationModel::mdl0config_product_name', and
        'Mdl0ApplicationModel::mdl0config_prod_namespace'.
        
        Use cases:
        An engineering project administrator has created a product ItemRevision. A manufacturing engineering user
        creates and maintains VariantRules that represent a set of configurations for prototype builds. The project
        administrator attaches these VariantRules to the product ItemRevision. Product engineering users can use
        operation 'getProductConfigurations' to review existing prototype configurations. The user can then chose one
        of the configurations to initialize variant configuration criteria with operation 'setVariantCriteria'.
        
        Exceptions:
        >This operation may raise 'Teamcenter::Soa::Server::ServiceException' wrapping around following Teamcenter
        errors:
        
        92002: The Product Name and Product Namespace must not be empty.
        """
        return cls.execute_soa_method(
            method_name='getProductConfigurations',
            library='ConfigFilterCriteria',
            service_date='2013_05',
            service_name='VariantManagement',
            params={'productName': productName, 'productNameSpace': productNameSpace, 'configuratorURL': configuratorURL},
            response_cls=VariantCriteriaResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessModeler._2007_06.RulesBasedFramework import RbfNameValue, ExecuteRbfRulesResponse
from tcsoa.gen.BusinessModeler._2007_06.Constants import PropertyConstantKey, TypeConstantValueResponse, TypeConstantKey, GlobalConstantValueResponse, PropertyConstantValueResponse
from typing import List
from tcsoa.utils import TcService


class ConstantsService(TcService):

    @classmethod
    def getPropertyConstantValues(cls, keys: List[PropertyConstantKey]) -> PropertyConstantValueResponse:
        """
        This operation gets the values of the named property constants ('keys').
        """
        return cls.execute_soa_method(
            method_name='getPropertyConstantValues',
            library='BusinessModeler',
            service_date='2007_06',
            service_name='Constants',
            params={'keys': keys},
            response_cls=PropertyConstantValueResponse,
        )

    @classmethod
    def getTypeConstantValues(cls, keys: List[TypeConstantKey]) -> TypeConstantValueResponse:
        """
        This operation gets the values of the named type constants ('keys').
        """
        return cls.execute_soa_method(
            method_name='getTypeConstantValues',
            library='BusinessModeler',
            service_date='2007_06',
            service_name='Constants',
            params={'keys': keys},
            response_cls=TypeConstantValueResponse,
        )

    @classmethod
    def getGlobalConstantValues(cls, keys: List[str]) -> GlobalConstantValueResponse:
        """
        Global constants provide consistent definitions that can be used throughout the system. These constants have
        one or multiple values.  User can retrieve the values of global constants to determine the system behavior
        based on values. This operation gets the values of the named global constants ('keys'). This operation only
        supports single valued global constants, for multivalued constants use the 'getGlobalConstantValues2' operation.
        """
        return cls.execute_soa_method(
            method_name='getGlobalConstantValues',
            library='BusinessModeler',
            service_date='2007_06',
            service_name='Constants',
            params={'keys': keys},
            response_cls=GlobalConstantValueResponse,
        )


class RulesBasedFrameworkService(TcService):

    @classmethod
    def executeRbfRules(cls, id: str, inputs: List[RbfNameValue]) -> ExecuteRbfRulesResponse:
        """
        This operation invokes the CLIPS rules engine to apply the set of application extension rules that belong to
        the specified application extension point for the specified input name/value pairs.  The result of the
        execution is returned in the output name/value pairs.
        """
        return cls.execute_soa_method(
            method_name='executeRbfRules',
            library='BusinessModeler',
            service_date='2007_06',
            service_name='RulesBasedFramework',
            params={'id': id, 'inputs': inputs},
            response_cls=ExecuteRbfRulesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanFile, BOMLine
from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateVisSCsFromBOMsInfo(TcBaseObj):
    """
    Input structure used for creating VisStrucutreContext objects based on the given BOMWindows and specific
    occurrences within those BOMWindows.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var occurrencesList: List of BOMLines representing the occurrences to be included in the structure recipe.
    :var staticStructureFile: IMANFile reference to the PLMXML static representation of the structure. If not supplied
    then the associated property of the VisStructureContext will not be set. [optional]
    :var createVisSCOptions: A list of options that are to be considered when creating VisStructureContext objects
    during the execution of this operation. This list of options is represented by a map of string option names to
    string option values. The values are represented as a list of strings. The size of this list is dependent on the
    particular option being defined.
    """
    clientId: str = ''
    occurrencesList: List[BOMLine] = ()
    staticStructureFile: ImanFile = None
    createVisSCOptions: StringKeyToStringVectorMap = None


"""
Map that contains option names and the corresponding option value(s).
"""
StringKeyToStringVectorMap = Dict[str, List[str]]



from tcsoa.gen.GlobalMultiSite._2007_06.services import SiteReservationService as imp0
from tcsoa.gen.GlobalMultiSite._2011_06.services import ImportExportService as imp1
from tcsoa.gen.GlobalMultiSite._2007_06.services import ImportExportService as imp2
from tcsoa.gen.GlobalMultiSite._2008_06.services import ImportExportService as imp3
from tcsoa.gen.GlobalMultiSite._2010_04.services import ImportExportService as imp4
from tcsoa.gen.GlobalMultiSite._2007_12.services import ImportExportService as imp5
from tcsoa.gen.GlobalMultiSite._2020_04.services import ImportExportService as imp6
from tcsoa.utils import TcService


class SiteReservationService(TcService):
    cancelSiteCheckOut = imp0.cancelSiteCheckOut
    siteCheckIn = imp0.siteCheckIn
    siteCheckOut = imp0.siteCheckOut


class ImportExportService(TcService):
    createGSIdentities = imp1.createGSIdentities
    createOrUpdateActionRules = imp2.createOrUpdateActionRules
    createOrUpdateClosureRules = imp2.createOrUpdateClosureRules
    createOrUpdateFilterRules = imp2.createOrUpdateFilterRules
    createOrUpdatePropertySets = imp2.createOrUpdatePropertySets
    createOrUpdateTransferModes = imp2.createOrUpdateTransferModes
    createOrUpdateTransferOptionSets = imp2.createOrUpdateTransferOptionSets
    exportObjectsToOfflinePackage = imp3.exportObjectsToOfflinePackage
    exportObjectsToPLMXML = imp4.exportObjectsToPLMXML
    getActionRules = imp2.getActionRules
    getAllTransferOptionSets = imp2.getAllTransferOptionSets
    getAvailableTransferOptionSets = imp2.getAvailableTransferOptionSets
    getClosureRules = imp2.getClosureRules
    getFilterRules = imp2.getFilterRules
    getHashedUID = imp1.getHashedUID
    getPropertySets = imp2.getPropertySets
    getRemoteSites = imp5.getRemoteSites
    getTransferModes = imp2.getTransferModes
    importObjectsFromOfflinePackage = imp3.importObjectsFromOfflinePackage
    importObjectsFromPLMXML = imp4.importObjectsFromPLMXML
    importObjectsFromPLMXMLWithDSM = imp6.importObjectsFromPLMXMLWithDSM
    requestExportToRemoteSites = imp5.requestExportToRemoteSites
    requestImportFromOfflinePackage = imp2.requestImportFromOfflinePackage
    requestImportFromRemoteSites = imp5.requestImportFromRemoteSites



from __future__ import annotations

from tcsoa.gen.Query._2010_09.SavedQuery import BusinessObjectQueryClause
from enum import Enum
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AttributeSortOption(TcBaseObj):
    """
    Structure for attribute sort option.
    
    :var sortAttribute: Attribute of the type being searched based on which the results will be sorted.
    :var sortOrder: Order in which results are sorted AscendingOrder or DescendingOrder.
    """
    sortAttribute: str = ''
    sortOrder: SortOrder = None


@dataclass
class BusinessObjectQueryInput3(TcBaseObj):
    """
    Input structure for executeBOQueriesWithSort operation.
    
    :var boTypeName: Name of business object type. Supports BusinessObject and all subtypes.
    :var clauses: Query clauses in search criteria. Each clause consists of property name, property value, logic and
    math operators.
    :var sortOptions: A list of attribute sort options based on which the results will be sorted. The first element of
    the list is the primary sort field, second is the secondary sort field and so on.
    :var maxNumToReturn: Specified maximum number of objects to return.
    :var requestId: Unique ID used to register the query execution task. This can be used by the caller to cancel the
    time consuming query, the value can be generated by any unique string generator. The query can be cancelled by
    calling: TcSoaFrameworkCore.Session.cancelOperations (requestId).
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure. This is currently not yet used by the return data elements, the caller can leave it empty.
    """
    boTypeName: str = ''
    clauses: List[BusinessObjectQueryClause] = ()
    sortOptions: List[AttributeSortOption] = ()
    maxNumToReturn: int = 0
    requestId: str = ''
    clientId: str = ''


class SortOrder(Enum):
    """
    Defines the order in which the business object query results are sorted (Ascending or Descending)
    """
    AscendingOrder = 'AscendingOrder'
    DescendingOrder = 'DescendingOrder'
